from aksui.ak import envelope
import hitbox

import gtk

class EnvelopeWidget(gtk.DrawingArea):
    def __init__(self, kg, index):
        # TODO: Enable scaling.
        gtk.DrawingArea.__init__(self)
        self.dragging = -1
        self.xoffset = 10
        self.yoffset = 10 
        self.set_size_request(250 + self.xoffset, 105 + self.yoffset)
        self.connect("expose_event", self.on_expose)
        self.connect("button_press_event", self.on_button_press)
        self.connect("button_release_event", self.on_button_release)
        self.connect("motion_notify_event", self.on_pointer_motion)
        self.rects = { }

        self.add_events(gtk.gdk.BUTTON_PRESS_MASK |
                        gtk.gdk.BUTTON_RELEASE_MASK |
                        gtk.gdk.POINTER_MOTION_MASK)

        self.set_envelope(kg, index)

    def set_envelope(self, kg, index):
        self.kg = kg
        self.index = index

        self.envelope = envelope.Envelope(kg, index)

        self.queue_draw()

    def on_pointer_motion(self, widget, event):
        if self.envelope:
            self.rects = { }
            points = self.getPoints()

            for x, y in points:
                hb = hitbox.HitBox(x-5, y-5, 10, 10)
                pindex = points.index([x, y])

                if hb.point_in(event.x, event.y) or pindex == self.dragging:
                    self.rects[pindex] = gtk.gdk.Rectangle(hb.x, hb.y, hb.w, hb.h) 

                if pindex == self.dragging:
                    deltax = event.x - self.start_point[0]
                    deltay = event.y - self.start_point[1]
                    self.envelope.updateNode(self.dragging, self.ratestart + deltax, self.levelstart - deltay)

            self.queue_draw()

    def on_button_release(self, widget, event):
        self.dragging = -1

    def on_button_press(self, widget, event):
        if self.envelope:
            points = self.getPoints()

            for x, y in points:
                hb = UI.HitBox(x-5, y-5, 10, 10)

                if hb.point_in(event.x, event.y): 
                    self.dragging = points.index([x, y])
                    self.start_point = [event.x, event.y]
                    pindex = self.dragging
                    rateattr = "envelope_rate" + str(pindex + 1)
                    levelattr = "envelope_level" + str(pindex + 1)
                    self.ratestart = getattr(self.envelope, rateattr)
                    self.levelstart = getattr(self.envelope, levelattr)

    def getPoints(self):
        if self.envelope:
            # 0 = amp, 1 = filt, 2 = aux
            points = [[
                        [self.envelope.envelope_rate1+self.xoffset, self.yoffset], 
                        [self.envelope.envelope_rate2 + self.envelope.envelope_rate1+self.xoffset, 100 - self.envelope.envelope_level2 + self.yoffset], 
                        [self.envelope.envelope_rate3 + self.envelope.envelope_rate2 + self.envelope.envelope_rate1 + self.xoffset, 100 + self.yoffset]
                     ], [
                        [self.envelope.envelope_rate1 + self.xoffset, 100 - self.envelope.envelope_level1 + self.yoffset], 
                        [self.envelope.envelope_rate2 + self.envelope.envelope_rate1 + self.xoffset, 100 - self.envelope.envelope_level2 + self.yoffset], 
                        [self.envelope.envelope_rate3 + self.envelope.envelope_rate2 + self.envelope.envelope_rate1 + self.xoffset, 100 - self.envelope.envelope_level3 + self.yoffset], 
                        [self.envelope.envelope_rate4 + self.envelope.envelope_rate3 + self.envelope.envelope_rate2 + self.envelope.envelope_rate1 + self.xoffset, 100 - self.envelope.envelope_level4 + self.yoffset]], [
                        [self.envelope.envelope_rate1 + self.xoffset, 100 - self.envelope.envelope_level1 + self.yoffset], 
                        [self.envelope.envelope_rate2 + self.envelope.envelope_rate1 + self.xoffset, 100 - self.envelope.envelope_level2 + self.yoffset], 
                        [self.envelope.envelope_rate3 + self.envelope.envelope_rate2 + self.envelope.envelope_rate1 + self.xoffset, 100 - self.envelope.envelope_level3 + self.yoffset], 
                        [self.envelope.envelope_rate4 + self.envelope.envelope_rate3 + self.envelope.envelope_rate2 + self.envelope.envelope_rate1 + self.xoffset, 100 - self.envelope.envelope_level4 + self.yoffset]]
                     ]

            i = 0
           
            return points[self.envelope.index]

    def on_expose(self, widget, event):
        if self.envelope:
            self.context = widget.window.cairo_create()
            self.context.set_source_rgb(1, 1, 1)
            self.context.fill_preserve()
            self.context.set_source_rgb(0, 0, 0)
            self.context.stroke()

            self.context.move_to(self.xoffset, 100+self.yoffset)

            points = self.getPoints()
            for x, y in points:
                self.context.line_to(x, y)
                #self.context.line_to(x,y)
                #self.context.line_to(x,y)
            self.context.stroke()

            if self.envelope.index == 0:
                sustpt = [self.envelope.envelope_rate2 + self.envelope.envelope_rate1, 100 - self.envelope.envelope_level2]
            else:
                sustpt = [self.envelope.envelope_rate3 + self.envelope.envelope_rate2 + self.envelope.envelope_rate1, 100 - self.envelope.envelope_level3]

            x, y = sustpt

            self.context.set_dash([0, 1, 0, 0, 1, 1], 0.5)
            self.context.move_to(x+self.xoffset, 0)
            self.context.line_to(x+self.xoffset, 100+self.yoffset)
            self.context.stroke()

            win = widget.window
            self.gc = win.new_gc()
            self.gc.foreground = win.get_colormap().alloc_color("blue")

            for point in self.rects:
                rect = self.rects[point]
                #self.context.rectangle(rect.x,rect.y,rect.width,rect.height)
                win.draw_rectangle(self.gc, False, rect.x, rect.y, rect.width, rect.height) 





class ModulationPin:
    def __init__(self, s, pin_index, source, dest, level, keygroup_index):
        self.s = s
        self.pin_index = pin_index
        self.source = source
        self.dest = dest
        self.level = level
        self.keygroup_index = keygroup_index
        self.min = -100
        self.max = 100

    def set_value(self, value, all = False):
        # keep it within bounds

        if value > self.max:
            value = self.max
        elif value < self.min:
            value = self.min

        self.level = value
       
        if all:
            keygroup_index = 0
        else:
            keygroup_index = self.keygroup_index
       
        #print "DEBUG", self.pin_index, keygroup_index, self.source, self.dest, self.level
        self.s.programtools.set_modulation_connection(self.pin_index, keygroup_index, self.source, self.dest, self.level)
        
        return self.level



import gtk

import panelbase, rangewidget

class KeygroupPanel(panelbase.PanelBase):
    def __init__(self, kg, cb):
        panelbase.PanelBase.__init__(self,kg,"Keygroup Details",cb)
        
    def setup(self, kg):
        self.clear_children(True)
        print kg.index
        
        self.kg = kg
        self.s = kg.s

        vbox = gtk.VBox()
        hbox = gtk.HBox()

        controls = [
            rangewidget.AkKnobWidget(kg, "level", -600, 60, 10, "db"),
            rangewidget.AkKnobWidget(kg, "tune", -3600, 3600, 100, ""),
            rangewidget.AkKnobWidget(kg, "polyphony", 1, 64, 1, "voices"),
            # mute group
            # fx send
            # send volume
            ]

        for control in controls:
            hbox.pack_start(control, False, False, 1)
        vbox.pack_start(hbox, False, False, 1)
        self.pack_start(vbox, False, False, 1)
        self.show_all()


from aksui.utils import modelutils

class MultiFX:
    fxlist = [ 'NO EFFECT', 'CHORUS>MONO', 'CHORUS>STEREO', 'CHORUS>XOVER', 'CHORUS+DELAY', 'COMPRESSOR/LIMITER', 'DELAY>MONO', 'DELAY>MULTI TAP', 'DELAY>PAN', 'DELAY>PING PONG', 'DELAY>STEREO', 'DELAY>XOVER', 'DIGITAL EQ', 'DISTORTION', 'ENHANCER', 'EXPANDER', 'FLANGER>MONO', 'FLANGER>PAN', 'FLANGER>STEREO', 'FLANGER>XOVER', 'FLANGER+DELAY', 'NOISE GATE', 'PAN>AUTO PAN', 'PAN>TRIGGER PAN', 'PHASER>MONO', 'PHASER>PAN', 'PHASER>STEREO', 'PHASER>XOVER', 'PHASER+DELAY', 'PITCH CORRECTOR', 'PITCH SHIFTER', 'REVERB>AUDITORIUM', 'REVERB>BIG HALL', 'REVERB>BIG ROOM', 'REVERB>BRIGHT HALL', 'REVERB>DRUM BOOTH', 'REVERB>LIVE HOUSE', 'REVERB>MEDIUM HALL', 'REVERB>MEDIUM ROOM', 'REVERB>NON LINEAR', 'REVERB>PLATE 1', 'REVERB>PLATE 2', 'REVERB>REVERSE', 'REVERB>SMALL HALL', 'REVERB>SMALL ROOM', 'REVERB>STUDIO', 'REVERB>THEATER', 'REVERB>VOCAL PLATE', 'REVERB>WAREHOUSE', 'ROTARY SPEAKER', 'TAPE ECHO', 'WAH>AUTO WAH', 'WAH>TOUCH WAH' ]

    def __init__(self,s):
        # store sampler
        if s:
            self.s = s
        else:
            self.s = None

        self.update()

        for ci in self.channels:
            channel = self.channels[ci]
            #channel.dump() 


    def update(self):
        if self.s:
            # on eb4js seems to be multiple channels and 1 module per channel
            self.channels = { }
            self.no_channels = self.s.multifxtools.get_no_channels()
            for ci in range(self.no_channels):
                self.channels[ci] = MultiFXChannel(self,ci)                

class MultiFXEffect:
    def __init__(self,name,id,param_index_output_ctrl):
        self.name = name
        self.id = id
        # parameter index for output control
        self.param_index_output_ctrl = param_index_output_ctrl

class MultiFXChannel:
    # parameters
    # effect name
    # which channel

    outputmap = ['off','L-R','1-2','3-4','5-6','7-8','L','R','1','2','3','4','5','6','7','8']
    inputmap = ['SEND A','SEND B','SEND C','SEND D','FX1','FX2','FX3','FX4']

    def __init__(self,mfx,index):
        self.mfx = mfx
        self.s = mfx.s
        s = self.s
        self.index = index
        self.max_modules = s.multifxtools.get_max_modules(index)
        self.muted = s.multifxtools.is_channel_muted(index)
        self.input = s.multifxtools.get_channel_input(index)
        self.output = s.multifxtools.get_channel_output(index)
        self.modules = { }


        for i in range(self.max_modules):
            self.modules[i] = MultiFXModule(self,i)

    def dump(self):
        attrs = ['index','muted','input','output']
        for attr in attrs:
            print 'Param ',self.index,' :',attr,' = ',getattr(self,attr)

        for mi in self.modules:
            module = self.modules[mi]
            module.dump()

    def updateInput(self,i):
        return self.s.multifxtools.set_channel_input(self.index,i)

    def updateOutput(self,i):
        return self.s.multifxtools.set_channel_output(self.index,i)
        
       
class MultiFXModule:
    def __init__(self,c,i):
        self.s = c.mfx.s
        s = c.mfx.s
        self.mfx = c.mfx
        self.channel = c
        self.index = i

        self.update_name()
        self.update()

    def setEffectById(self,id):
        c = self.channel
        tools = self.s.multifxtools

        ret = tools.set_fx_by_id(c.index,self.index,id)

        self.update_name()
        self.update()
        return ret

    def enable(self):
        c = self.channel
        tools = self.s.multifxtools
        return tools.enable_module(c.index,self.index,True)

    def disable(self):
        c = self.channel
        tools = self.s.multifxtools
        return tools.enable_module(c.index,self.index,False)

    def update_name(self):
        i,s,c =self.index,self.s,self.channel

        self.effect_name = s.multifxtools.get_by_name(c.index,i)
        self.effect_index = s.multifxtools.get_by_index(c.index,i)
        
    def update(self):
        c = self.channel
        s = c.mfx.s
        i = self.index

        self.enabled = s.multifxtools.is_module_enabled(c.index,i)
        self.number_of_parameters = s.multifxtools.get_number_of_parameters(c.index,i)
        self.parameters = { }

        for pi in range(self.number_of_parameters):
            self.parameters[pi] = MultiFXParam(s,c,self,pi)

    def dump(self):
        attrs = ['effect_name','enabled','number_of_parameters']
        for attr in attrs:
            print '\tParam ',self.index,' :',attr,' = ',getattr(self,attr)
        for pi in self.parameters:
            param = self.parameters[pi]
            param.dump()

class MultiFXParam:
    def __init__(self,s,channel,module,index):
        self.channel = channel
        self.s = channel.s
        self.module = module
        self.index = index

        ci = self.channel.index
        mi = self.module.index
        pi = self.index

        self.maximum = s.multifxtools.get_parameter_maximum(ci,mi,pi)
        self.minimum = s.multifxtools.get_parameter_minimum(ci,mi,pi)
        self.name = s.multifxtools.get_parameter_name(ci,mi,pi)
        self.units = s.multifxtools.get_parameter_units(ci,mi,pi)
        self.type = s.multifxtools.get_parameter_type(ci,mi,pi)
        self.template = s.multifxtools.get_display_template(ci,mi,pi)
        self.position_id = s.multifxtools.get_parameter_position_id(ci,mi,pi)
        self.value = s.multifxtools.get_param_value(ci,mi,pi)
        self.string = s.multifxtools.get_param_string(ci,mi,pi)
        self.qlink = s.multifxtools.get_param_qlinkctrl(ci,mi,pi)

        formatmap = {None:"%(value)d%(units)s",'':"%(value)d%(units)s"," 0.0":"%(value).1f%(units)s","  0.00":"%(value).2f%(units)s","   0":"%(value)d%(units)s","  0":"%(value)d%(units)s"}
        self.format = formatmap[self.template]

    def format_value(self):
        s = self.s
        ci = self.channel.index
        mi = self.module.index
        pi = self.index

        value = self.value

        if self.template == " 0.0":
            value /= 10
        elif self.template == "  0.00":
            value /= 100

        if self.string:
            formatted = self.string 
        else:
            formatted = self.format % {"value":value, "units":self.units}
       
        return formatted

    def on_value_changed(self, adj, lblFormat = None):
        s,ci,mi,pi = self.s,self.channel.index,self.module.index,self.index
        s.multifxtools.set_param_value(ci,mi,pi,int(adj.value))

        self.value = adj.value

        if self.string:
            self.string = s.multifxtools.get_param_string(ci,mi,pi)

        lblFormat.set_text(self.format_value())

    def dump(self):
        attrs = ['maximum','minimum','name','type','position_id']
        for attr in attrs:
            print '\t\tParam ',self.index,' :',attr,' = ',getattr(self,attr)

inputmapmodel = modelutils.get_model_from_list(MultiFXChannel.inputmap)
outputmapmodel = modelutils.get_model_from_list(MultiFXChannel.outputmap)
effectmodel = modelutils.get_model_from_list(MultiFX.fxlist)



import gtk
import os.path

import panelbase, rangewidget
from aksui.utils import midiutils

class ZonePanel(panelbase.PanelBase):
    def __init__(self,kg,cb):
        self.dnd_list = [ ( 'text/uri-list', 0, 80 ) ] 
        panelbase.PanelBase.__init__(self,kg,"Zones",cb)
        

    def on_drag_data_received(self, widget, context, x, y, selection, target_type, timestamp, kg, zone_index, cb):
        self.s.FileChooser.on_drag_data_received(widget, context, x, y, selection, target_type, timestamp)
        # if it was a single selection, set that zone
        if len(self.s.FileChooser.files) > 0 and kg:
            first_file = os.path.basename(self.s.FileChooser.files[0])
            (filename, ext) = first_file.split(".")
            kg.zones[zone_index].set("sample", filename)
        # hack, this should be updating rather than completely refreshing
        cb.set_model(self.s.samplesmodel)
        cb.somodel = self.s.samplesmodel
        cb.init()

    def setup(self, kg):
        self.clear_children(True)
        
        self.kg = kg
        self.s = kg.s
        zonevbox = gtk.VBox()
        self.sample_combos = []

        for j in range(4):
            zone = kg.zones[j]
            zone.set_callback = self.set_callback
            #zonewidget = MiniZoneWidget(kg.zones[j])

            #zonewidget = AkComboBox(zone, "sample", z48.samplesmodel)
            zonehbox = gtk.HBox()
            self.sample_combos.append(rangewidget.AkComboBox(zone, "sample", self.s.samplesmodel, False))
            zonecombos = [
                self.sample_combos[j],
# TODO:                rangewidget.AkComboBox(zone, "output", utils.sampler_lists["output"]),
# TODO:                rangewidget.AkComboBox(zone, "keyboard_track", utils.sampler_lists["keyboard_track"]),
                rangewidget.AkComboBox(zone, "playback", midiutils.sampler_lists["playback_b"])
                ]
            zoneknobs = [
                rangewidget.AkKnobWidget(zone, "level", -600, 60, 10, "db"),
                rangewidget.AkKnobWidget(zone, "pan", 0, 100, 1, ""),
                rangewidget.AkKnobWidget(zone, "filter", -100, 100, 1, ""),
                rangewidget.AkKnobWidget(zone, "mod_start", -9999, 9999, 1, ""),
                rangewidget.AkKnobWidget(zone, "tune", -3600, 3600, 100, ""),
                ]

            for zonecombo in zonecombos:
                zonehbox.pack_start(zonecombo, True, True, 1)
                zonecombo.drag_dest_set(gtk.DEST_DEFAULT_MOTION | gtk.DEST_DEFAULT_HIGHLIGHT | gtk.DEST_DEFAULT_DROP, self.dnd_list, gtk.gdk.ACTION_COPY)
                zonecombo.connect("drag_data_received", self.on_drag_data_received, kg, j, zonecombo)

            #zonevbox.pack_start(zonehbox, False, False, 0)
                
            #zonehbox = gtk.HBox()
            for zoneknob in zoneknobs:
                zonehbox.pack_start(zoneknob, False, False, 1)

            zonevbox.pack_start(zonehbox, False, False, 2)

        self.pack_start(zonevbox)
        self.show_all()




""" Python equivalent of akai section recordingtools

Methods to facilitate recording
"""

__author__ =  'Walco van Loon'
__version__ =  '0.2'

from aksy.devices.akai.sysex import Command

import aksy.devices.akai.sysex_types

class Recordingtools:
    def __init__(self, z48):
        self.sampler = z48
        self.get_status_cmd = Command('_', '\x30\x01', 'recordingtools', 'get_status', (), None)
        self.get_progress_cmd = Command('_', '\x30\x02', 'recordingtools', 'get_progress', (), None)
        self.get_max_rec_time_cmd = Command('_', '\x30\x03', 'recordingtools', 'get_max_rec_time', (), None)
        self.arm_cmd = Command('_', '\x30\x10', 'recordingtools', 'arm', (), None)
        self.start_cmd = Command('_', '\x30\x11', 'recordingtools', 'start', (), None)
        self.stop_cmd = Command('_', '\x30\x12', 'recordingtools', 'stop', (), None)
        self.cancel_cmd = Command('_', '\x30\x13', 'recordingtools', 'cancel', (), None)
        self.start_playing_cmd = Command('_', '\x30\x20', 'recordingtools', 'start_playing', (), None)
        self.stop_playing_cmd = Command('_', '\x30\x21', 'recordingtools', 'stop_playing', (), None)
        self.keep_cmd = Command('_', '\x30\x22', 'recordingtools', 'keep', (), None)
        self.delete_cmd = Command('_', '\x30\x23', 'recordingtools', 'delete', (), None)
        self.set_input_cmd = Command('_', '\x32\x01', 'recordingtools', 'set_input', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_mode_cmd = Command('_', '\x32\x02', 'recordingtools', 'set_mode', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.enable_monitor_cmd = Command('_', '\x32\x03', 'recordingtools', 'enable_monitor', (aksy.devices.akai.sysex_types.BOOL,), None)
        self.set_rec_time_cmd = Command('_', '\x32\x04', 'recordingtools', 'set_rec_time', (aksy.devices.akai.sysex_types.DWORD,), None)
        self.set_orig_pitch_cmd = Command('_', '\x32\x05', 'recordingtools', 'set_orig_pitch', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_threshold_cmd = Command('_', '\x32\x06', 'recordingtools', 'set_threshold', (aksy.devices.akai.sysex_types.SBYTE,), None)
        self.set_trigger_src_cmd = Command('_', '\x32\x07', 'recordingtools', 'set_trigger_src', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_bit_depth_cmd = Command('_', '\x32\x08', 'recordingtools', 'set_bit_depth', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_prerec_time_cmd = Command('_', '\x32\x09', 'recordingtools', 'set_prerec_time', (aksy.devices.akai.sysex_types.WORD,), None)
        self.set_dest_cmd = Command('_', '\x32\x0A', 'recordingtools', 'set_dest', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_name_cmd = Command('_', '\x32\x10', 'recordingtools', 'set_name', (aksy.devices.akai.sysex_types.STRING,), None)
        self.set_name_seed_cmd = Command('_', '\x32\x11', 'recordingtools', 'set_name_seed', (aksy.devices.akai.sysex_types.STRING,), None)
        self.set_autorec_mode_cmd = Command('_', '\x32\x12', 'recordingtools', 'set_autorec_mode', (aksy.devices.akai.sysex_types.BOOL,), None)
        self.set_autonormalize_cmd = Command('_', '\x32\x13', 'recordingtools', 'set_autonormalize', (aksy.devices.akai.sysex_types.BOOL,), None)
        self.get_input_cmd = Command('_', '\x33\x01', 'recordingtools', 'get_input', (), None)
        self.get_mode_cmd = Command('_', '\x33\x02', 'recordingtools', 'get_mode', (), None)
        self.get_monitor_cmd = Command('_', '\x33\x03', 'recordingtools', 'get_monitor', (), None)
        self.get_rec_time_cmd = Command('_', '\x33\x04', 'recordingtools', 'get_rec_time', (), None)
        self.get_pitch_cmd = Command('_', '\x33\x05', 'recordingtools', 'get_pitch', (), None)
        self.get_threshold_cmd = Command('_', '\x33\x06', 'recordingtools', 'get_threshold', (), None)
        self.get_trigger_src_cmd = Command('_', '\x33\x07', 'recordingtools', 'get_trigger_src', (), None)
        self.get_bit_depth_cmd = Command('_', '\x33\x08', 'recordingtools', 'get_bit_depth', (), None)
        self.get_prerec_time_cmd = Command('_', '\x33\x09', 'recordingtools', 'get_prerec_time', (), None)
        self.get_dest_cmd = Command('_', '\x33\x0A', 'recordingtools', 'get_dest', (), None)
        self.get_name_cmd = Command('_', '\x33\x10', 'recordingtools', 'get_name', (), None)
        self.get_name_seed_cmd = Command('_', '\x33\x11', 'recordingtools', 'get_name_seed', (), None)
        self.get_autorec_mode_cmd = Command('_', '\x33\x12', 'recordingtools', 'get_autorec_mode', (), None)
        self.get_autonormalize_cmd = Command('_', '\x33\x13', 'recordingtools', 'get_autonormalize', (), None)

    def get_status(self):
        """Get Record Status

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_status_cmd, ())

    def get_progress(self):
        """Get Record Progress

        Returns:
            DWORD
        """
        return self.sampler.execute(self.get_progress_cmd, ())

    def get_max_rec_time(self):
        """Get Maximum Record Time

        Returns:
            DWORD
        """
        return self.sampler.execute(self.get_max_rec_time_cmd, ())

    def arm(self):
        """Arm Recording
        """
        return self.sampler.execute(self.arm_cmd, ())

    def start(self):
        """Start Recording
        """
        return self.sampler.execute(self.start_cmd, ())

    def stop(self):
        """Stop Recording
        """
        return self.sampler.execute(self.stop_cmd, ())

    def cancel(self):
        """Cancel Recording
        """
        return self.sampler.execute(self.cancel_cmd, ())

    def start_playing(self):
        """Play Recorded Sample Start
        """
        return self.sampler.execute(self.start_playing_cmd, ())

    def stop_playing(self):
        """Play Recorded Sample Stop
        """
        return self.sampler.execute(self.stop_playing_cmd, ())

    def keep(self):
        """Keep Recorded Sample. Sample with name assigned above, is added to the list of available samples.
        """
        return self.sampler.execute(self.keep_cmd, ())

    def delete(self):
        """Delete Recorded Sample
        """
        return self.sampler.execute(self.delete_cmd, ())

    def set_input(self, arg0):
        """Set Input <Data1> = (0=ANALOGUE, 1=DIGITAL, 2=MAIN OUT 3=ADAT1/2, 4=ADAT3/4, 5=ADAT5/6, 6=ADAT7/8)
        """
        return self.sampler.execute(self.set_input_cmd, (arg0, ))

    def set_mode(self, arg0):
        """Set Record Mode <Data1> = (0=STEREO, 1=MONO L, 2=MONO R, 3=L/R MIX)
        """
        return self.sampler.execute(self.set_mode_cmd, (arg0, ))

    def enable_monitor(self, arg0):
        """Set Record Monitor
        """
        return self.sampler.execute(self.enable_monitor_cmd, (arg0, ))

    def set_rec_time(self, arg0):
        """Set Record Time <Data1> = time in seconds. If <Data1> = 0, MANUAL mode is enabled.
        """
        return self.sampler.execute(self.set_rec_time_cmd, (arg0, ))

    def set_orig_pitch(self, arg0):
        """Set Original Pitch
        """
        return self.sampler.execute(self.set_orig_pitch_cmd, (arg0, ))

    def set_threshold(self, arg0):
        """Set Threshold <Data1> = threshold in dB (-63,0)
        """
        return self.sampler.execute(self.set_threshold_cmd, (arg0, ))

    def set_trigger_src(self, arg0):
        """Set Trigger Source (0=OFF, 1=AUDIO, 2=MIDI)
        """
        return self.sampler.execute(self.set_trigger_src_cmd, (arg0, ))

    def set_bit_depth(self, arg0):
        """Set Bit Depth <Data1> = (0=16-bit, 1=24-bit)
        """
        return self.sampler.execute(self.set_bit_depth_cmd, (arg0, ))

    def set_prerec_time(self, arg0):
        """Set Pre-recording Time <Data1> = time in ms
        """
        return self.sampler.execute(self.set_prerec_time_cmd, (arg0, ))

    def set_dest(self, arg0):
        """Set Recording Detination <Data1> = (0=RAM, 1=DISK) Note: seems to be unsupported
        """
        return self.sampler.execute(self.set_dest_cmd, (arg0, ))

    def set_name(self, arg0):
        """Set Record Name
        """
        return self.sampler.execute(self.set_name_cmd, (arg0, ))

    def set_name_seed(self, arg0):
        """Set Record Name Seed
        """
        return self.sampler.execute(self.set_name_seed_cmd, (arg0, ))

    def set_autorec_mode(self, arg0):
        """Set Auto-Record Mode <Data1> = (0=OFF, 1=ON)
        """
        return self.sampler.execute(self.set_autorec_mode_cmd, (arg0, ))

    def set_autonormalize(self, arg0):
        """Set Auto-Normalise Mode <Data1> = (0=OFF, 1=ON)
        """
        return self.sampler.execute(self.set_autonormalize_cmd, (arg0, ))

    def get_input(self):
        """Get Input (0=ANALOGUE, 1=DIGITAL, 2=MAIN OUT, 3=ADAT1/2, 4=ADAT3/4, 5=ADAT5/6, 6=ADAT7/8)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_input_cmd, ())

    def get_mode(self):
        """Get Record Mode (0=STEREO, 1=MONO L, 2=MONO R, 3=L/R MIX)
        """
        return self.sampler.execute(self.get_mode_cmd, ())

    def get_monitor(self):
        """Get Record Monitor <Reply> = (0=OFF, 1=ON)

        Returns:
            BOOL
        """
        return self.sampler.execute(self.get_monitor_cmd, ())

    def get_rec_time(self):
        """Get Record Time <Reply> = time in seconds.

        Returns:
            DWORD
        """
        return self.sampler.execute(self.get_rec_time_cmd, ())

    def get_pitch(self):
        """Get Original Pitch

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_pitch_cmd, ())

    def get_threshold(self):
        """Get Threshold <Reply> = threshold in dB -63,0

        Returns:
            SBYTE
        """
        return self.sampler.execute(self.get_threshold_cmd, ())

    def get_trigger_src(self):
        """Get Trigger Source <Reply> = (0=OFF, 1=AUDIO, 2=MIDI)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_trigger_src_cmd, ())

    def get_bit_depth(self):
        """Get Bit Depth <Reply> = (0=16-bit, 1=24-bit)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_bit_depth_cmd, ())

    def get_prerec_time(self):
        """Get Pre-recording Time <Reply> = time in ms

        Returns:
            WORD
        """
        return self.sampler.execute(self.get_prerec_time_cmd, ())

    def get_dest(self):
        """Get Recording Destination <Reply> = (0=RAM, 1=DISK) Note: seems to be unsupported

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_dest_cmd, ())

    def get_name(self):
        """Get Record Name

        Returns:
            STRING
        """
        return self.sampler.execute(self.get_name_cmd, ())

    def get_name_seed(self):
        """Get Record Name Seed

        Returns:
            STRING
        """
        return self.sampler.execute(self.get_name_seed_cmd, ())

    def get_autorec_mode(self):
        """Get Auto-Record Mode <Reply> = (0=OFF, 1=ON)

        Returns:
            BOOL
        """
        return self.sampler.execute(self.get_autorec_mode_cmd, ())

    def get_autonormalize(self):
        """Get Auto-Normalise Mode <Reply> = (0=OFF, 1=ON)

        Returns:
            BOOL
        """
        return self.sampler.execute(self.get_autonormalize_cmd, ())





""" Python equivalent of akai section systemtools

Methods to manipulate system parameters
"""

__author__ =  'Walco van Loon'
__version__ =  '0.2'

from aksy.devices.akai.sysex import Command

import aksy.devices.akai.sysex_types

class Systemtools:
    def __init__(self, z48):
        self.sampler = z48
        self.get_os_software_version_cmd = Command('_', '\x04\x00', 'systemtools', 'get_os_software_version', (), None)
        self.get_os_subversion_cmd = Command('_', '\x04\x01', 'systemtools', 'get_os_subversion', (), None)
        self.get_sampler_model_cmd = Command('_', '\x04\x04', 'systemtools', 'get_sampler_model', (), None)
        self.get_supported_filetypes_cmd = Command('_', '\x04\x08', 'systemtools', 'get_supported_filetypes', (), None)
        self.get_perc_free_wave_mem_cmd = Command('_', '\x04\x10', 'systemtools', 'get_perc_free_wave_mem', (), None)
        self.get_perc_free_cpu_mem_cmd = Command('_', '\x04\x11', 'systemtools', 'get_perc_free_cpu_mem', (), None)
        self.get_wave_mem_size_cmd = Command('_', '\x04\x12', 'systemtools', 'get_wave_mem_size', (), None)
        self.get_free_wave_mem_size_cmd = Command('_', '\x04\x13', 'systemtools', 'get_free_wave_mem_size', (), None)
        self.clear_sampler_mem_cmd = Command('_', '\x04\x18', 'systemtools', 'clear_sampler_mem', (), None)
        self.purge_unused_cmd = Command('_', '\x04\x19', 'systemtools', 'purge_unused', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.tag_unused_cmd = Command('_', '\x04\x1A', 'systemtools', 'tag_unused', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.compact_wave_mem_cmd = Command('_', '\x04\x20', 'systemtools', 'compact_wave_mem', (), None)
        self.cancel_compact_wave_mem_cmd = Command('_', '\x04\x21', 'systemtools', 'cancel_compact_wave_mem', (), None)
        self.get_compact_wave_mem_progress_cmd = Command('_', '\x04\x22', 'systemtools', 'get_compact_wave_mem_progress', (), None)
        self.get_async_operation_state_cmd = Command('_', '\x04\x30', 'systemtools', 'get_async_operation_state', (), None)
        self.cancel_curr_async_operation_cmd = Command('_', '\x04\x31', 'systemtools', 'cancel_curr_async_operation', (), None)
        self.get_sampler_name_cmd = Command('_', '\x07\x01', 'systemtools', 'get_sampler_name', (), None)
        self.get_scsi_id_cmd = Command('_', '\x07\x02', 'systemtools', 'get_scsi_id', (), None)
        self.get_master_tune_cmd = Command('_', '\x07\x03', 'systemtools', 'get_master_tune', (), None)
        self.get_master_level_cmd = Command('_', '\x07\x04', 'systemtools', 'get_master_level', (), None)
        self.get_midi_mode_cmd = Command('_', '\x07\x05', 'systemtools', 'get_midi_mode', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.is_qlink_local_ctrl_enabled_cmd = Command('_', '\x07\x06', 'systemtools', 'is_qlink_local_ctrl_enabled', (), None)
        self.is_default_items_enabled_cmd = Command('_', '\x07\x07', 'systemtools', 'is_default_items_enabled', (), None)
        self.get_midi_file_save_format_cmd = Command('_', '\x07\x08', 'systemtools', 'get_midi_file_save_format', (), None)
        self.get_cdr_write_speed_cmd = Command('_', '\x07\x09', 'systemtools', 'get_cdr_write_speed', (), None)
        self.get_cdr_write_mode_cmd = Command('_', '\x07\x0A', 'systemtools', 'get_cdr_write_mode', (), None)
        self.is_front_panel_locked_cmd = Command('_', '\x07\x10', 'systemtools', 'is_front_panel_locked', (), None)
        self.get_display_contrast_cmd = Command('_', '\x07\x11', 'systemtools', 'get_display_contrast', (), None)
        self.get_note_display_cmd = Command('_', '\x07\x12', 'systemtools', 'get_note_display', (), None)
        self.get_date_format_cmd = Command('_', '\x07\x13', 'systemtools', 'get_date_format', (), None)
        self.get_time_format_cmd = Command('_', '\x07\x14', 'systemtools', 'get_time_format', (), None)
        self.get_waveform_view_scale_cmd = Command('_', '\x07\x18', 'systemtools', 'get_waveform_view_scale', (), None)
        self.get_waveform_view_type_cmd = Command('_', '\x07\x19', 'systemtools', 'get_waveform_view_type', (), None)
        self.get_waveform_view_fill_cmd = Command('_', '\x07\x1A', 'systemtools', 'get_waveform_view_fill', (), None)
        self.get_item_sort_mode_cmd = Command('_', '\x07\x1B', 'systemtools', 'get_item_sort_mode', (), None)
        self.get_year_cmd = Command('_', '\x07\x20', 'systemtools', 'get_year', (), None)
        self.get_month_cmd = Command('_', '\x07\x21', 'systemtools', 'get_month', (), None)
        self.get_day_cmd = Command('_', '\x07\x22', 'systemtools', 'get_day', (), None)
        self.get_day_of_week_cmd = Command('_', '\x07\x23', 'systemtools', 'get_day_of_week', (), None)
        self.get_hours_cmd = Command('_', '\x07\x24', 'systemtools', 'get_hours', (), None)
        self.get_mins_cmd = Command('_', '\x07\x25', 'systemtools', 'get_mins', (), None)
        self.get_secs_cmd = Command('_', '\x07\x26', 'systemtools', 'get_secs', (), None)
        self.get_system_clock_cmd = Command('_', '\x07\x30', 'systemtools', 'get_system_clock', (), None)
        self.get_dig_sync_cmd = Command('_', '\x07\x31', 'systemtools', 'get_dig_sync', (), None)
        self.get_dig_format_cmd = Command('_', '\x07\x32', 'systemtools', 'get_dig_format', (), None)
        self.get_adat_main_out_cmd = Command('_', '\x07\x33', 'systemtools', 'get_adat_main_out', (), None)
        self.get_play_mode_cmd = Command('_', '\x07\x40', 'systemtools', 'get_play_mode', (), None)
        self.get_prog_monitor_mode_cmd = Command('_', '\x07\x41', 'systemtools', 'get_prog_monitor_mode', (), None)
        self.get_sample_monitor_mode_cmd = Command('_', '\x07\x42', 'systemtools', 'get_sample_monitor_mode', (), None)
        self.get_play_key_note_cmd = Command('_', '\x07\x48', 'systemtools', 'get_play_key_note', (), None)
        self.get_play_key_velocity_cmd = Command('_', '\x07\x49', 'systemtools', 'get_play_key_velocity', (), None)
        self.get_play_key_midi_channel_cmd = Command('_', '\x07\x4a', 'systemtools', 'get_play_key_midi_channel', (), None)
        self.get_play_key_echo_cmd = Command('_', '\x07\x4b', 'systemtools', 'get_play_key_echo', (), None)
        self.get_prog_change_enable_cmd = Command('_', '\x07\x4c', 'systemtools', 'get_prog_change_enable', (), None)
        self.get_autoload_enable_cmd = Command('_', '\x07\x4d', 'systemtools', 'get_autoload_enable', (), None)
        self.get_global_pad_mode_cmd = Command('_', '\x07\x50', 'systemtools', 'get_global_pad_mode', (), None)
        self.get_pad_midi_channel_cmd = Command('_', '\x07\x51', 'systemtools', 'get_pad_midi_channel', (), None)
        self.get_pad_sensitivity_cmd = Command('_', '\x07\x52', 'systemtools', 'get_pad_sensitivity', (), None)
        self.get_def_note_assign_cmd = Command('_', '\x07\x53', 'systemtools', 'get_def_note_assign', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.get_chrom_start_note_cmd = Command('_', '\x07\x54', 'systemtools', 'get_chrom_start_note', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_sampler_name_cmd = Command('_', '\x06\x01', 'systemtools', 'set_sampler_name', (aksy.devices.akai.sysex_types.STRING,), None)
        self.set_scsi_id_cmd = Command('_', '\x06\x02', 'systemtools', 'set_scsi_id', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_master_tune_cmd = Command('_', '\x06\x03', 'systemtools', 'set_master_tune', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_master_level_cmd = Command('_', '\x06\x04', 'systemtools', 'set_master_level', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_midi_out_thru_cmd = Command('_', '\x06\x05', 'systemtools', 'set_midi_out_thru', (), None)
        self.set_qlink_local_control_cmd = Command('_', '\x06\x06', 'systemtools', 'set_qlink_local_control', (aksy.devices.akai.sysex_types.BOOL,), None)
        self.set_create_default_items_cmd = Command('_', '\x06\x07', 'systemtools', 'set_create_default_items', (aksy.devices.akai.sysex_types.BOOL,), None)
        self.set_midi_file_save_format_cmd = Command('_', '\x06\x08', 'systemtools', 'set_midi_file_save_format', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_cdr_write_speed_cmd = Command('_', '\x06\x09', 'systemtools', 'set_cdr_write_speed', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_cdr_write_mode_cmd = Command('_', '\x06\x0a', 'systemtools', 'set_cdr_write_mode', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_front_panel_lockout_state_cmd = Command('_', '\x06\x10', 'systemtools', 'set_front_panel_lockout_state', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_display_contrast_cmd = Command('_', '\x06\x11', 'systemtools', 'set_display_contrast', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_note_display_cmd = Command('_', '\x06\x12', 'systemtools', 'set_note_display', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_date_display_format_cmd = Command('_', '\x06\x13', 'systemtools', 'set_date_display_format', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_time_display_format_cmd = Command('_', '\x06\x14', 'systemtools', 'set_time_display_format', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_waveform_view_scale_cmd = Command('_', '\x06\x18', 'systemtools', 'set_waveform_view_scale', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_waveform_view_type_cmd = Command('_', '\x06\x19', 'systemtools', 'set_waveform_view_type', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_waveform_view_fill_cmd = Command('_', '\x06\x1a', 'systemtools', 'set_waveform_view_fill', (aksy.devices.akai.sysex_types.BOOL,), None)
        self.set_item_sort_mode_cmd = Command('_', '\x06\x1b', 'systemtools', 'set_item_sort_mode', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_year_cmd = Command('_', '\x06\x20', 'systemtools', 'set_year', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_month_cmd = Command('_', '\x06\x21', 'systemtools', 'set_month', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_day_cmd = Command('_', '\x06\x22', 'systemtools', 'set_day', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_day_of_week_cmd = Command('_', '\x06\x23', 'systemtools', 'set_day_of_week', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_hours_cmd = Command('_', '\x06\x24', 'systemtools', 'set_hours', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_minutes_cmd = Command('_', '\x06\x25', 'systemtools', 'set_minutes', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_seconds_cmd = Command('_', '\x06\x26', 'systemtools', 'set_seconds', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_system_clock_cmd = Command('_', '\x06\x30', 'systemtools', 'set_system_clock', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_digital_out_sync_cmd = Command('_', '\x06\x31', 'systemtools', 'set_digital_out_sync', (), None)
        self.set_digital_format_cmd = Command('_', '\x06\x32', 'systemtools', 'set_digital_format', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_adat_main_out_cmd = Command('_', '\x06\x33', 'systemtools', 'set_adat_main_out', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_play_mode_cmd = Command('_', '\x06\x40', 'systemtools', 'set_play_mode', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_program_monitor_mode_cmd = Command('_', '\x06\x41', 'systemtools', 'set_program_monitor_mode', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_sample_monitor_mode_cmd = Command('_', '\x06\x42', 'systemtools', 'set_sample_monitor_mode', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_play_key_note_cmd = Command('_', '\x06\x48', 'systemtools', 'set_play_key_note', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_play_key_velocity_cmd = Command('_', '\x06\x49', 'systemtools', 'set_play_key_velocity', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_play_key_midi_channel_cmd = Command('_', '\x06\x4a', 'systemtools', 'set_play_key_midi_channel', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_play_key_echo_cmd = Command('_', '\x06\x4b', 'systemtools', 'set_play_key_echo', (aksy.devices.akai.sysex_types.BOOL,), None)
        self.set_program_change_enable_cmd = Command('_', '\x06\x4c', 'systemtools', 'set_program_change_enable', (aksy.devices.akai.sysex_types.BOOL,), None)
        self.set_autoload_enable_cmd = Command('_', '\x06\x4d', 'systemtools', 'set_autoload_enable', (aksy.devices.akai.sysex_types.BOOL,), None)
        self.set_global_pad_mode_cmd = Command('_', '\x05\x50', 'systemtools', 'set_global_pad_mode', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_pad_midi_channel_cmd = Command('_', '\x06\x51', 'systemtools', 'set_pad_midi_channel', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_pad_sensitivity_cmd = Command('_', '\x06\x52', 'systemtools', 'set_pad_sensitivity', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)
        self.set_default_note_assignment_cmd = Command('_', '\x06\x53', 'systemtools', 'set_default_note_assignment', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)
        self.set_chromatic_start_note_cmd = Command('_', '\x06\x54', 'systemtools', 'set_chromatic_start_note', (aksy.devices.akai.sysex_types.BYTE,), None)

    def get_os_software_version(self):
        """Get Operating System Software Version

        Returns:
            BYTE
            BYTE
        """
        return self.sampler.execute(self.get_os_software_version_cmd, ())

    def get_os_subversion(self):
        """Get the Sub-Version of the Operating System

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_os_subversion_cmd, ())

    def get_sampler_model(self):
        """Get Sampler Model  (0=Z4, 1=Z8, 2=MPC4000)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_sampler_model_cmd, ())

    def get_supported_filetypes(self):
        """Get List of supported filetypes [returns DONE message]

        Returns:
            STRINGARRAY
        """
        return self.sampler.execute(self.get_supported_filetypes_cmd, ())

    def get_perc_free_wave_mem(self):
        """Get the percentage free Wave memory

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_perc_free_wave_mem_cmd, ())

    def get_perc_free_cpu_mem(self):
        """Get the percentage free CPU memory

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_perc_free_cpu_mem_cmd, ())

    def get_wave_mem_size(self):
        """Get the total number of kilobytes of Wave memory

        Returns:
            DWORD
        """
        return self.sampler.execute(self.get_wave_mem_size_cmd, ())

    def get_free_wave_mem_size(self):
        """Get the number of kilobytes of free Wave memory

        Returns:
            DWORD
        """
        return self.sampler.execute(self.get_free_wave_mem_size_cmd, ())

    def clear_sampler_mem(self):
        """Clear Sampler Memory (delete all items from memory)
        """
        return self.sampler.execute(self.clear_sampler_mem_cmd, ())

    def purge_unused(self, arg0):
        """Purge Unused Items <Data1> = (0=SAMPLE, 1=PROGRAM)
        """
        return self.sampler.execute(self.purge_unused_cmd, (arg0, ))

    def tag_unused(self, arg0):
        """Tag Unused Items <Data1> = (0=SAMPLE, 1=PROGRAM)
        """
        return self.sampler.execute(self.tag_unused_cmd, (arg0, ))

    def compact_wave_mem(self):
        """Start Compact Wave Memory
        """
        return self.sampler.execute(self.compact_wave_mem_cmd, ())

    def cancel_compact_wave_mem(self):
        """Cancel Compact Wave Memory
        """
        return self.sampler.execute(self.cancel_compact_wave_mem_cmd, ())

    def get_compact_wave_mem_progress(self):
        """Get Compact Wave Memory Progress (%)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_compact_wave_mem_progress_cmd, ())

    def get_async_operation_state(self):
        """Get State of Asynchronous Operation ERROR 'operation is pending' or DONE

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_async_operation_state_cmd, ())

    def cancel_curr_async_operation(self):
        """Cancel Current Asynchronous Operation
        """
        return self.sampler.execute(self.cancel_curr_async_operation_cmd, ())

    def get_sampler_name(self):
        """Get Sampler Name

        Returns:
            STRING
        """
        return self.sampler.execute(self.get_sampler_name_cmd, ())

    def get_scsi_id(self):
        """Get SCSI self ID

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_scsi_id_cmd, ())

    def get_master_tune(self):
        """Get Master Tune

        Returns:
            SWORD
        """
        return self.sampler.execute(self.get_master_tune_cmd, ())

    def get_master_level(self):
        """Get Master Level <Reply> = (-42 dB - 0dB in 6dB steps)(0=-42 dB, 1=-36dB, ..., 7=0dB)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_master_level_cmd, ())

    def get_midi_mode(self, arg0):
        """Get MIDI OUT/THRU <Data1> = MIDI port (0=A, 1=B) <Reply> = (0=OUT, 1=THRUA, 2=THRUB)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_midi_mode_cmd, (arg0, ))

    def is_qlink_local_ctrl_enabled(self):
        """Get Qlink Local Control <Reply> = (0=OFF, 1=ON)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.is_qlink_local_ctrl_enabled_cmd, ())

    def is_default_items_enabled(self):
        """Get Create Default Items at Startup <Reply> = (0=OFF, 1=ON)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.is_default_items_enabled_cmd, ())

    def get_midi_file_save_format(self):
        """Get MIDI file save format

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_midi_file_save_format_cmd, ())

    def get_cdr_write_speed(self):
        """Get CD-R write speed (0=*1, 1=*2, 2=*4, 3=*6, 4=*8, 5=*12, 6=*16, 7=MAX)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_cdr_write_speed_cmd, ())

    def get_cdr_write_mode(self):
        """Get CD-R write mode <Reply> = (0=TEST+WRITE, 1=TEST ONLY, 2=WRITE ONLY)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_cdr_write_mode_cmd, ())

    def is_front_panel_locked(self):
        """Get Front panel lock-out state

        Returns:
            BYTE
        """
        return self.sampler.execute(self.is_front_panel_locked_cmd, ())

    def get_display_contrast(self):
        """Get Display Contrast

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_display_contrast_cmd, ())

    def get_note_display(self):
        """Get Note Display <Reply> = (0=NUMBER, 1=NAME)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_note_display_cmd, ())

    def get_date_format(self):
        """Get Date Display Format  <Reply> = (0=DDMMYY, 1=MMDDYY, 2=YYMMDD)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_date_format_cmd, ())

    def get_time_format(self):
        """Get Time Display Format <Reply> = (0=12HOUR, 1=24HOUR)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_time_format_cmd, ())

    def get_waveform_view_scale(self):
        """Get Waveform View Scale <Reply> = (0=LINEAR, 1=LOG)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_waveform_view_scale_cmd, ())

    def get_waveform_view_type(self):
        """Get Waveform View Type <Reply> = (0=RECTIFIED, 1=BIPOLAR)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_waveform_view_type_cmd, ())

    def get_waveform_view_fill(self):
        """Get Waveform View Fill <Reply> = (0=OFF, 1=ON)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_waveform_view_fill_cmd, ())

    def get_item_sort_mode(self):
        """Get Item Sort Mode <Reply> = (0=ALPHABETIC, 1=MEMORY)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_item_sort_mode_cmd, ())

    def get_year(self):
        """Get Year

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_year_cmd, ())

    def get_month(self):
        """Get Month

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_month_cmd, ())

    def get_day(self):
        """Get Day

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_day_cmd, ())

    def get_day_of_week(self):
        """Get Day of Week (0=SUN)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_day_of_week_cmd, ())

    def get_hours(self):
        """Get Hours

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_hours_cmd, ())

    def get_mins(self):
        """Get Minutes

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_mins_cmd, ())

    def get_secs(self):
        """Get Seconds

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_secs_cmd, ())

    def get_system_clock(self):
        """Get System Clock <Reply> = (0=44.1kHz, 1=48kHz, 2=96kHz)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_system_clock_cmd, ())

    def get_dig_sync(self):
        """Get Digital Out Sync (0=INTERNAL, 1=DIGITAL IN, 2=ADAT IN, 3=WORDCLOCK)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_dig_sync_cmd, ())

    def get_dig_format(self):
        """Get Digital Format <Reply> = (0=PRO, 1=CONSUMER)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_dig_format_cmd, ())

    def get_adat_main_out(self):
        """Get ADAT Main Out <Reply> = (0=L/R, 1=1/2)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_adat_main_out_cmd, ())

    def get_play_mode(self):
        """Get Play Mode (0=Multi, 1=Program; 2=Sample; 3=Muted), handle of item which is the active Play Item

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_play_mode_cmd, ())

    def get_prog_monitor_mode(self):
        """Get Program Monitor Mode  (0=Multi, 1=Program(OMNI))

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_prog_monitor_mode_cmd, ())

    def get_sample_monitor_mode(self):
        """Get Sample Monitor Mode (0=Multi, 1=Program; 2=Sample(OMNI))

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_sample_monitor_mode_cmd, ())

    def get_play_key_note(self):
        """Get Play Key Note

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_play_key_note_cmd, ())

    def get_play_key_velocity(self):
        """Get Play Key Velocity

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_play_key_velocity_cmd, ())

    def get_play_key_midi_channel(self):
        """Get Play Key Midi Channel <Reply> = (1A=0, 2A=1, ..., 16B=31)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_play_key_midi_channel_cmd, ())

    def get_play_key_echo(self):
        """Get Play Key Echo <Reply> = (0=OFF, 1=ON)

        Returns:
            BOOL
        """
        return self.sampler.execute(self.get_play_key_echo_cmd, ())

    def get_prog_change_enable(self):
        """Get Program Change Enable <Reply> = (0=OFF, 1=ON)

        Returns:
            BOOL
        """
        return self.sampler.execute(self.get_prog_change_enable_cmd, ())

    def get_autoload_enable(self):
        """Get Autoload Enable <Reply> = (0=OFF, 1=ON)

        Returns:
            BOOL
        """
        return self.sampler.execute(self.get_autoload_enable_cmd, ())

    def get_global_pad_mode(self):
        """Get Global Pad Mode <Reply> = (0=DEFAULT, 1=CHROMATIC)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_global_pad_mode_cmd, ())

    def get_pad_midi_channel(self):
        """Get MIDI Channel for MPC Pad

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_pad_midi_channel_cmd, ())

    def get_pad_sensitivity(self):
        """Get Pad Sensitivity <Data1> = Pad <Reply> = Sensitivity (0-100 = 100%-200%)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_pad_sensitivity_cmd, ())

    def get_def_note_assign(self, arg0):
        """Get Default Note Assignment <Data1> = Pad

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_def_note_assign_cmd, (arg0, ))

    def get_chrom_start_note(self, arg0):
        """Get Default Note Assignment <Data1> = Pad

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_chrom_start_note_cmd, (arg0, ))

    def set_sampler_name(self, arg0):
        """Set Sampler Name
        """
        return self.sampler.execute(self.set_sampler_name_cmd, (arg0, ))

    def set_scsi_id(self, arg0):
        """Set SCSI ID
        """
        return self.sampler.execute(self.set_scsi_id_cmd, (arg0, ))

    def set_master_tune(self, arg0):
        """Set Master Tune
        """
        return self.sampler.execute(self.set_master_tune_cmd, (arg0, ))

    def set_master_level(self, arg0):
        """Set Master Level <Data1> = (-42dB - 0dB in 6dB * 7 steps)
        """
        return self.sampler.execute(self.set_master_level_cmd, (arg0, ))

    def set_midi_out_thru(self):
        """Set MIDI OUT/THRU <Data1> = MIDI port (0=A, 1=B), <Data2> = (0=OUT, 1=THRUA, 2=THRUB)
        """
        return self.sampler.execute(self.set_midi_out_thru_cmd, ())

    def set_qlink_local_control(self, arg0):
        """Set Qlink Local Control
        """
        return self.sampler.execute(self.set_qlink_local_control_cmd, (arg0, ))

    def set_create_default_items(self, arg0):
        """Set Create Default Items at Startup
        """
        return self.sampler.execute(self.set_create_default_items_cmd, (arg0, ))

    def set_midi_file_save_format(self, arg0):
        """Set MIDI file save format
        """
        return self.sampler.execute(self.set_midi_file_save_format_cmd, (arg0, ))

    def set_cdr_write_speed(self, arg0):
        """Set CD-R write speed <Data1> = (0=*1, 1=*2, 2=*4, 3=*6, 4=*8, 5=*12, 6=*16, 7=MAX)
        """
        return self.sampler.execute(self.set_cdr_write_speed_cmd, (arg0, ))

    def set_cdr_write_mode(self, arg0):
        """Set CD-R write mode <Data1> = (0=TEST+WRITE, 1=TEST ONLY, 2=WRITE ONLY)
        """
        return self.sampler.execute(self.set_cdr_write_mode_cmd, (arg0, ))

    def set_front_panel_lockout_state(self, arg0):
        """Set Front panel lock-out state <Data1> = (0=NORMAL; 1=LOCKED)
        """
        return self.sampler.execute(self.set_front_panel_lockout_state_cmd, (arg0, ))

    def set_display_contrast(self, arg0):
        """  Set Display Contrast
        """
        return self.sampler.execute(self.set_display_contrast_cmd, (arg0, ))

    def set_note_display(self, arg0):
        """Set Note Display <Data1> = (0=NUMBER, 1=NAME)
        """
        return self.sampler.execute(self.set_note_display_cmd, (arg0, ))

    def set_date_display_format(self, arg0):
        """Set Date Display Format <Data1> = (0=DDMMYY, 1=MMDDYY, 2=YYMMDD)
        """
        return self.sampler.execute(self.set_date_display_format_cmd, (arg0, ))

    def set_time_display_format(self, arg0):
        """Set Time Display Format <Data1> = (0=12HOUR, 1=24HOUR)
        """
        return self.sampler.execute(self.set_time_display_format_cmd, (arg0, ))

    def set_waveform_view_scale(self, arg0):
        """Set Waveform View Scale <Data1> = (0=LINEAR, 1=LOG)
        """
        return self.sampler.execute(self.set_waveform_view_scale_cmd, (arg0, ))

    def set_waveform_view_type(self, arg0):
        """Set Waveform View Type <Data1> = (0=RECTIFIED, 1=BIPOLAR)
        """
        return self.sampler.execute(self.set_waveform_view_type_cmd, (arg0, ))

    def set_waveform_view_fill(self, arg0):
        """Set Waveform View Fill <Data1> = (0=OFF, 1=ON)
        """
        return self.sampler.execute(self.set_waveform_view_fill_cmd, (arg0, ))

    def set_item_sort_mode(self, arg0):
        """Set Item Sort Mode <Data1> = (0=ALPHABETIC, 1=MEMORY)
        """
        return self.sampler.execute(self.set_item_sort_mode_cmd, (arg0, ))

    def set_year(self, arg0):
        """Set Year
        """
        return self.sampler.execute(self.set_year_cmd, (arg0, ))

    def set_month(self, arg0):
        """Set Month
        """
        return self.sampler.execute(self.set_month_cmd, (arg0, ))

    def set_day(self, arg0):
        """Set Day
        """
        return self.sampler.execute(self.set_day_cmd, (arg0, ))

    def set_day_of_week(self, arg0):
        """Set Day of Week (0=SUN)
        """
        return self.sampler.execute(self.set_day_of_week_cmd, (arg0, ))

    def set_hours(self, arg0):
        """Set Hours
        """
        return self.sampler.execute(self.set_hours_cmd, (arg0, ))

    def set_minutes(self, arg0):
        """Set Minutes
        """
        return self.sampler.execute(self.set_minutes_cmd, (arg0, ))

    def set_seconds(self, arg0):
        """Set Seconds
        """
        return self.sampler.execute(self.set_seconds_cmd, (arg0, ))

    def set_system_clock(self, arg0):
        """Set System Clock <Data1> = (0=44.1kHz, 1=48kHz, 2=96kHz)
        """
        return self.sampler.execute(self.set_system_clock_cmd, (arg0, ))

    def set_digital_out_sync(self):
        """Set Digital Out Sync <Data1> = (0=INTERNAL, 1=DIGITAL IN, 2=ADAT IN, 3=WORDCLOCK) BYTE
        """
        return self.sampler.execute(self.set_digital_out_sync_cmd, ())

    def set_digital_format(self, arg0):
        """Set Digital Format <Data1> = (0=PRO, 1=CONSUMER)
        """
        return self.sampler.execute(self.set_digital_format_cmd, (arg0, ))

    def set_adat_main_out(self, arg0):
        """Set ADAT Main Out <Data1> = (0=L/R, 1=1/2)
        """
        return self.sampler.execute(self.set_adat_main_out_cmd, (arg0, ))

    def set_play_mode(self, arg0):
        """Set Play Mode <Data1> = (0=Multi, 1=Program; 2=Sample; 3=Muted) <Data2> = handle of item to become active Play Item
        """
        return self.sampler.execute(self.set_play_mode_cmd, (arg0, ))

    def set_program_monitor_mode(self, arg0):
        """Set Program Monitor Mode <Data1> = (0=Multi, 1=Program(OMNI))
        """
        return self.sampler.execute(self.set_program_monitor_mode_cmd, (arg0, ))

    def set_sample_monitor_mode(self, arg0):
        """Set Sample Monitor Mode <Data1> = (0=Multi, 1=Program; 2=Sample(OMNI))
        """
        return self.sampler.execute(self.set_sample_monitor_mode_cmd, (arg0, ))

    def set_play_key_note(self, arg0):
        """Set Play Key Note
        """
        return self.sampler.execute(self.set_play_key_note_cmd, (arg0, ))

    def set_play_key_velocity(self, arg0):
        """Set Play Key Velocity
        """
        return self.sampler.execute(self.set_play_key_velocity_cmd, (arg0, ))

    def set_play_key_midi_channel(self, arg0):
        """Set Play Key Midi Channel <Data1> = (1A=0, 2A=1, ..., 16B=31)
        """
        return self.sampler.execute(self.set_play_key_midi_channel_cmd, (arg0, ))

    def set_play_key_echo(self, arg0):
        """Set Play Key Echo <Data1> = (0=OFF, 1=ON)
        """
        return self.sampler.execute(self.set_play_key_echo_cmd, (arg0, ))

    def set_program_change_enable(self, arg0):
        """Set Program Change Enable <Data1> = (0=OFF, 1=ON)
        """
        return self.sampler.execute(self.set_program_change_enable_cmd, (arg0, ))

    def set_autoload_enable(self, arg0):
        """Set Autoload Enable <Data1> = (0=OFF, 1=ON)
        """
        return self.sampler.execute(self.set_autoload_enable_cmd, (arg0, ))

    def set_global_pad_mode(self, arg0):
        """Set Global Pad Mode <Data1> = (0=DEFAULT, 1=CHROMATIC)
        """
        return self.sampler.execute(self.set_global_pad_mode_cmd, (arg0, ))

    def set_pad_midi_channel(self, arg0):
        """Set MIDI Channel
        """
        return self.sampler.execute(self.set_pad_midi_channel_cmd, (arg0, ))

    def set_pad_sensitivity(self, arg0, arg1):
        """Set Pad Sensitivity <Data1> = Pad <Data2> = Sensitivity (0-100 = 100%-200%)
        """
        return self.sampler.execute(self.set_pad_sensitivity_cmd, (arg0, arg1, ))

    def set_default_note_assignment(self, arg0, arg1):
        """Set Default Note Assignment <Data1> = Pad, <Data2> = Note
        """
        return self.sampler.execute(self.set_default_note_assignment_cmd, (arg0, arg1, ))

    def set_chromatic_start_note(self, arg0):
        """Set Chromatic Start Note
        """
        return self.sampler.execute(self.set_chromatic_start_note_cmd, (arg0, ))




#!/usr/bin/python

import os, os.path, fnmatch, sys 

from aksy.device import Devices
from aksy import config

__author__ = 'Walco van Loon'
__version__ = '0.01'

def create_option_parser(): 
    usage = """%prog [filename patterns]
    
    Download all program files and multis from memory:
    
    %prog *.akp *.akm
    
    Download the AUTOLOAD folder from disk:
    
    %prog -l disk sampler*/AUTOLOAD"""
    parser = config.create_option_parser(usage=usage)
    parser.add_option("-l", nargs=1, default="memory", dest="location",
          help="Download from location (memory or disk)")
    parser.add_option("-d", nargs=1, default=".", dest="destdir",
          help="The destination directory")
    parser.add_option("-o", default=False, dest="overwriteExisting",
          help="Whether to overwrite existing files")
    return parser

def download_from_memory(sampler, destdir, patterns, overwrite):
    collected = [item.get_name() for item in sampler.memory.get_children()]
    matched = []
    for pat in patterns:
        matched.extend(fnmatch.filter(collected, pat))
    
    if len(matched) == 0:
        return

    ensure_destdir(destdir)
    
    for name in matched:
        download(sampler, destdir, name, sampler.MEMORY, overwrite)

def download_from_disk(sampler, destdir, patterns, overwrite):
    dir_patterns = patterns[0].split('/')
    download_children(sampler, sampler.disks, destdir, PatternYielder(dir_patterns), overwrite)

def download(sampler, destdir, name, location, overwrite):
    destfile = os.path.join(destdir, name)
    if os.path.exists(destfile) and not overwrite:
        print "Skipping existing file ", destfile
    else:
        print "Downloading %s to %s" % (name, destfile)
        sampler.transfertools.get(name, destfile)

def download_children(sampler, parent, destdir, patternYielder, overwrite):
    pat = patternYielder.next()
    filtered = [child for child in parent.get_children() if fnmatch.fnmatch(child.get_name(), pat)]

    fullpath = os.path.join(destdir, parent.get_name())

    if len(filtered) > 0:
        ensure_destdir(fullpath)
        
    for child in filtered:
        if child.has_children():
            download_children(sampler, child, fullpath, patternYielder, overwrite)
        else:
            download(sampler, fullpath, child.get_name(), sampler.DISK, overwrite)

def main():
    parser = create_option_parser()
    (options, patterns) = parser.parse_args()

    if len(patterns) == 0:
        patterns.append("*.wav")
    
    options.destdir = os.path.abspath(options.destdir)

    sampler = Devices.get_instance(options.sampler_type, options.connector)
    try:    
        execute_cmd(sampler, patterns, options)
    finally:
        sampler.close()

def execute_cmd(sampler, patterns, options):
    if options.location == "memory":
        download_from_memory(sampler, options.destdir, patterns, options.overwriteExisting)
    elif options.location == "disk":
        download_from_disk(sampler, options.destdir, patterns, options.overwriteExisting)
    else:
        parser.error("Invalid location: %s" % options.location)


def ensure_destdir(destdir):
    if not os.path.exists(destdir):
        os.mkdir(destdir)
    
class PatternYielder:
    def __init__(self, patterns):
        self.patterns = patterns
        self.index = -1
    def next(self):
        try:
            self.index += 1
            print self.patterns[self.index]
            return self.patterns[self.index]
        except IndexError:
            return "*"




import gtk

import base, zoneeditor, keygroupdetails, keygroupfilter, keygroupenvelope
from aksui.ak import keygroup

class KeygroupEditor(base.Base):
    def __init__(self, program, index):
        base.Base.__init__(self, None, "notebookKeygroup")

        self.s = program.s
        self.p = program
        self.index = index

        self.zonesInitialized = False

        self.filter = None
        self.details = None

        self.set_keygroup(index)
        
        self.editor.append_page(self.envelopes.editor, gtk.Label("Envelopes"))

    def update_zone(self, zonename, zone):
        z = getattr(self, zonename)
        z.set_zone(zone)
        z.editor.show_all()

    def update_zones(self):
        self.update_zone("zone1",self.keygroup.zones[0]) 
        self.update_zone("zone2",self.keygroup.zones[1]) 
        self.update_zone("zone3",self.keygroup.zones[2]) 
        self.update_zone("zone4",self.keygroup.zones[3]) 

    def init_zones(self):
        if not self.zonesInitialized:
            self.zone1 = zoneeditor.ZoneEditor(self.keygroup.zones[0])
            self.zone2 = zoneeditor.ZoneEditor(self.keygroup.zones[1])
            self.zone3 = zoneeditor.ZoneEditor(self.keygroup.zones[2])
            self.zone4 = zoneeditor.ZoneEditor(self.keygroup.zones[3])
            self.w_tableZones.attach(self.zone1.editor,0,1,0,1)
            self.w_tableZones.attach(self.zone2.editor,1,2,0,1)
            self.w_tableZones.attach(self.zone3.editor,2,3,0,1)
            self.w_tableZones.attach(self.zone4.editor,3,4,0,1)
            self.zonesInitialized = True
       

    def set_keygroup(self, index):
        if index >= 0:
            self.keygroup = keygroup.Keygroup(self.p, index)

            if self.filter:
                self.w_expanderKeygroupFilter.remove(self.filter.editor)
            if self.details:
                self.w_expanderKeygroupDetails.remove(self.details.editor)

            self.details = keygroupdetails.KeygroupDetails(self.keygroup)
            self.filter = keygroupfilter.KeygroupFilter(self.keygroup)
            self.envelopes = keygroupenvelope.KeygroupEnvelopes(self.keygroup, None)

            self.w_expanderKeygroupFilter.add(self.filter.editor)
            self.w_expanderKeygroupDetails.add(self.details.editor)
            # XXX this is currently broken; need to pass in index here
            self.envelopes.update_env('ampenv', self.keygroup.amp_envelope)
            self.envelopes.update_env('filtenv', self.keygroup.filter_envelope)
            self.envelopes.update_env('auxenv', self.keygroup.aux_envelope)

            self.init_zones()
            self.update_zones()

            self.updating = False
            self.update()
        else:
            self.keygroup = None
            if hasattr(self,'ampenv'):
                self.ampenv.envelope = None
            if hasattr(self, 'filtenv'):
                self.filtenv.envelope = None
            if hasattr(self, 'auxenv'):
                self.auxenv.envelope = None

    def on_button_press_event(self, widget, event):
        if event.state & gtk.gdk.CONTROL_MASK:
            widget.set_value(0)


import samplerobject, program, keygroup, modulationmatrix, modulationpin

class Program(samplerobject.SamplerObject):
    def __init__(self, s, name, handle = None):
        samplerobject.SamplerObject.__init__(self, s, None, "programtools")

        self.name = name
        if handle:
            self.handle = handle
        else:
            self.handle = self.gettools().get_handle_by_name(self.name)

        self.specialattrs = ["name", ]

        self.attrs = ["name", "type", "group_id", "genre", "program_no", "no_keygroups", "keygroup_xfade", "keygroup_xfade_type", "level", "polyphony", "portamento_enabled", "portamento_mode", "portamento_time", "glissando_mode", "aftertouch_mode", "aftertouch_value", "reassignment_method", "softpedal_loudness_reduction", "softpedal_attack_stretch", "softpedal_filter_close", "midi_transpose", "tune", "legato", "pitchbend_up", "pitchbend_down", "pitchbend_mode", "no_modulation_connections"]
        self.attrs_minimal = ["name", "type", "no_keygroups", "level", "polyphony"]
        
        if self.name:
            self.s.programtools.set_curr_by_name(self.name)
        else:
            print "No name..."
            
        self.precache()
        
    def get_handle(self):
        return self.handle
        
    def copy(self, destination_name):
        tools = self.gettools()
        tools.set_curr_by_name(self.name)
        tools.copy_program(destination_name)
        return program.Program(self.s, destination_name)
        
    def init_recycled(self):
        # ultimately this should be a plugin or something, but for now...
        # updates a recycle generated program to the settings i prefer use
        for kg in self.get_keygroups():
            for zone in kg.zones:
                zone.set("playback", 1) # ONE SHOT
            kg.set("polyphony", 1)

    def get_keygroups(self):
        keygroups = []

        for i in range(self.no_keygroups):
            keygroups.append(keygroup.Keygroup(self, i))

        return keygroups

    def set_name(self, name):
        self.s.programtools.set_curr_by_name(self.name)
        self.s.programtools.rename_curr(name)
        self.attrscache["name"] = name

    def dump_matrix(self):
        result = []
        n = self.s.programtools.get_no_modulation_connections()
        #print "number of modulation connections", n
        result.append("TotalNumber of modulation connections for %s: %d" % (self.name, n))
        for i in range(1, self.s.programtools.get_no_keygroups() + 1):
            result.append('Keygroup %i' % i)
            conns = self.get_matrix(i)

            result.append("----")
        
            for c in conns[0]:
                if c.source > 0:
                    print c.source, c.dest, c.level
                    result.append("%d. %s => %s = %d" % (c.pin_index, modulationmatrix.ModulationMatrix.sources[c.source], modulationmatrix.ModulationMatrix.destinations[c.dest], c.level))
                
        return '\n'.join(result)
    
    def get_matrix(self, keygroup_index):
        assert keygroup_index > 0, "0 (ALL) is not supported"
        
        n = self.s.programtools.get_no_modulation_connections()

        pins_dict = {}
        pins = []

        cmd = self.s.programtools.get_modulation_connection_cmd
        cmds = [cmd] * 8

        for j in range(8):
            # create batch get command
            args = []
            # ????: can only do 8 at a time?
            for i in range(8*j, 8*(j+1)):
                #print "getting mod connection:", keygroup_index, "/", i
                args.append([i, keygroup_index])

            results = self.s.execute_alt_request(self.handle, cmds, args)

            # NOTE: the following unpacking only works if single levels are returned (i.e. for specific keygroups) 
            total_results = len(results) / 3

            for i in range(total_results):
                offset = 3*i
                (source, dest, level) = results[offset:offset+3] # snag 3 at a time
                pin = modulationpin.ModulationPin(self.s, i, source, dest, level, keygroup_index)
                pins.append(pin)
                pins_dict[(source, dest)] = pin
            
        return (pins, pins_dict)



"""aksy model

Offers a high level sampler API

"""
import os.path, logging
from aksyx import AkaiSampler
from aksy import fileutils


handlers = {}
LOG = logging.getLogger("aksy.model")

def register_handlers(tools):
    """Initialize the handlers, keyed on class
    definitions.
    """
    handlers.update(tools)

class SamplerException(Exception):
    """ Exception raised by the sampler
    """
    def __init(self, msg, code):
        Exception.__init__(self, msg)
        self.code = code

    def get_code(self):
        return self.code

class Container(object):
    def get_children(self):
        raise NotImplementedError()

    def get_child(self, name):
        for child in self.get_children():
            if child.get_name() == name:
                return child
        return None

class Disk(Container):
    def __init__(self, (handle, disk_type, format, scsi_id, writable, name)):
        self._handle = handle
        # type: 0=floppy; 1=hard disk; 2=CD ROM; 3=removable disk.
        self._disk_type = disk_type
        # format, where: 0=other; 1=MSDOS; 2=FAT32; 3=ISO9660; 4=S1000; 5=S3000; 6=EMU; 7=ROLAND, 
        # 8=CD-AUDIO, 100=EMPTY
        self._format = format
        self._scsi_id = scsi_id
        self._writable = writable
        self._name = name
        self.root = Folder("")

    def has_children(self):
        try:
            self.set_current()
            return self.root.has_children()
        except SamplerException:
            return False

    def get_handle(self):
        return self._handle

    def is_writable(self):
        return self._writable
    
    def set_current(self):
        handlers[Disk].select_disk(self.get_handle())

    def get_name(self):
        return self._name
    
    def get_short_name(self):
        return self.get_name()
    
    def get_size(self):
        return None

    def get_modified(self):
        return False
    
    def create_folder(self, name):
        self.root.create_folder(name)

    def get_children(self):
        self.set_current()
        return self.root.get_children()
    
    def refresh(self):
        self.root.refresh()
        
    def get_actions(self):
        return ('upload',)
    
def get_file_type(name):
    assert name is not None
    
    if fileutils.is_multi(name):
        return Multi, FileRef.MULTI
    if fileutils.is_program(name):
        return Program, FileRef.PROGRAM
    if fileutils.is_sample(name):
        return Sample, FileRef.SAMPLE
    if fileutils.is_song(name):
        return Song, FileRef.SONG
    
    msg = "No support for file type: %s", name
    LOG.exception(msg)
    raise NotImplementedError(msg)
    
class FileRef(object):
    FOLDER = 0
    MULTI = 1
    PROGRAM = 2
    SAMPLE = 3
    SONG = 4

    def __init__(self, path, size=None):
        """Initializes a file object - A multi, program or sample before it
        is loaded into memory
        """
        self.path = path
        self.size = size
        self.type = get_file_type(self.get_name())[1]

    def get_name(self):
        return os.path.basename(self.path)
    
    def get_short_name(self):
        """ Returns name without extension
        """
        return os.path.splitext(self.get_name())[0]

    def get_handle(self):
        """Returns handle for the file
        """
        return self.path

    def get_size(self):
        return self.size

    def get_modified(self):
        """Returns True if this file has been modified
        """
        return True

    def has_children(self):
        return False

    def copy(self, dest_path):
        """Copies a file
        """
        item = self.load()
        new_parent = Folder(dest_path)

        return new_parent.append_child(item)

    def load(self):
        """Load the file into memory
        """
        self.get_parent().set_current()
        handlers[Disk].load_file(self.get_name())
        return InMemoryFile.get_instance(self.get_name())

    def download(self, path):
        """download the file to host
        """
        LOG.info("download of file %s to %s" % (self.get_name(), repr(path)))
        # XXX: remove the reference to the sampler
        handlers[Disk].z48.transfertools.get(self.get_name(), path)

    def get_parent(self):
        return Folder(os.path.dirname(self.path))

    def delete(self):
        self.get_parent().set_current()
        handlers[Disk].delete(self.get_name())

    def rename(self, new_name):
        self.get_parent().set_current()
        handlers[Disk].rename_file(self.get_name(), new_name)
        self.path = os.path.join(os.path.dirname(self.path, new_name))

    def get_actions(self):
        return ('load', 'delete', 'download',)

class Folder(FileRef, Container):
    def __init__(self, path):
        self.path = path
        self.type = FileRef.FOLDER
        self.children = []
        self.writable = True

    def set_writable(self, writable):
        self.writable = writable
        
    def is_writable(self):
        return self.writable
    
    def refresh(self):
        del self.children[:]

    def get_actions(self):
        return ('load', 'delete', 'download',)

    def get_children(self):
        """Gets the children of this folder
        or returns a cached version when already retrieved.
        """
        if len(self.children) > 0:
            return self.children

        self.set_current()
        folder_names = handlers[Disk].get_folder_names()
        if folder_names:
            self.children = [Folder(os.path.join(self.path, folder_name))
                for folder_name in folder_names if fileutils.is_valid_name(folder_name)]

        file_names_sizes = handlers[Disk].get_filenames()
        if file_names_sizes:
            self.children.extend(self.get_filename_sizes(file_names_sizes))
        return self.children

    def get_filename_sizes(self, names_sizes):
        for i in range(0, len(names_sizes), 2):
            name, size = names_sizes[i], names_sizes[i+1]
            if fileutils.is_file(name):
                yield FileRef(os.path.join(self.path, name), size)

    def get_child(self, name):
        for child in self.get_children():
            if child.get_name() == name:
                return child
        return None
    
    def has_children(self):
        return (len(self.children) > 0 or handlers[Disk].get_no_files() > 0 or
            handlers[Disk].get_no_folders() > 0)

    def get_name(self):
        return os.path.basename(self.path)

    def set_current(self):
        handlers[Disk].open_folder('')
        path = self.path.replace('/', '\\')
        handlers[Disk].open_folder(path)
        if LOG.isEnabledFor(logging.DEBUG):
            LOG.debug("set_current to: %s" % path)

    def copy(self, dest_path, recursive=True):
        """Copies a folder, default is including all its children
        """
        # copy the folder
        # copy the children to the new path

    def rename(self, new_name):
        self.get_parent().set_current()
        handlers[Disk].rename_folder(self.get_name(), new_name)
        self.path = self.path.replace(self.get_name(), new_name)

    def load(self):
        self.get_parent().set_current()
        handlers[Disk].load_folder(self.get_name())
        LOG.debug("Loading folder children %s" % repr (self.get_children()))
        return [item for item in self.get_children()]

    def create_folder(self, name):
        self.get_parent().set_current()
        handlers[Disk].create_folder(name)
        folder = Folder(os.path.join(self.path, name))
        self.children.append(folder)
        return folder

    def upload(self, path):
        self.set_current()
        name = os.path.basename(path)
        handlers[Disk].z48.put(path, name, destination=AkaiSampler.DISK)
        item = FileRef(self.path + (name,))
        self.children.append(item)
        return item

    def append_child(self, item):
        """Adds a child item to this folder
        Returns the added item
        """
        self.set_current()
        handlers[Disk].save(item.get_handle(), item.type, True, False)
        item = FileRef(self.path + (item.name,))
        self.children.append(item)
        return item

    def download(self, path):
        """download the folder to host
        """
        self.get_parent().set_current()
        path = os.path.join(path, self.get_name())
        LOG.info("download to dir: %s" % repr(path))
        if not os.path.exists(path):
            os.makedirs(path)
            for item in self.get_children():
                LOG.debug("download to dir: %s" % repr(path))
                item.download(os.path.join(path, item.get_name()))

class InMemoryFile(FileRef):
    @staticmethod
    def get_instance(name):
        clazz = get_file_type(name)[0]
        base_name = os.path.splitext(name)[0]
        handle = handlers[clazz].get_handle_by_name(base_name)
        return clazz(base_name, handle)

    def __cmp__(self, item):
        return cmp(self.get_short_name(), item.get_short_name())
    
    def __init__(self, name, handle):
        self.name = name
        self.handle = handle
        FileRef.__init__(self, (name,))

    def get_actions(self):
        return ('delete', 'download', 'copy')
    
    def get_name(self):
        return self.name
    
    def get_short_name(self):
        """ Returns name without extension
        """
        return os.path.splitext(self.get_name())[0]

    def get_handle(self):
        """Returns the handle
        """
        return self.handle

    def get_modified(self):
        self.set_current()
        return handlers[self.__class__].get_modified()
    
    def set_current(self):
        handlers[self.__class__].set_curr_by_name(self.get_short_name())

    def delete(self):
        LOG.info("InMemoryFile.delete() %s" % repr(self.get_name()))
        handlers[self.__class__].get_no_items()
        self.set_current()
        handlers[self.__class__].delete_curr()

    def save(self, overwrite, children=False):
        handlers[Disk].save(self.get_handle(), self.type, 
                                 overwrite, children)

    def download(self, dest_path):
        pass
    
    def copy(self, new_name):
        handlers[self.__class__].copy(new_name)

    def rename(self, new_name):
        self.set_current()
        handlers[self.__class__].rename_curr(new_name)

class Multi(InMemoryFile):
    def __init__(self, name, handle):
        InMemoryFile.__init__(self, name, handle)
        self.type = FileRef.MULTI

    def get_name(self):
        return InMemoryFile.get_name(self) + ".akm"

class Program(InMemoryFile):
    def __init__(self, name, handle):
        InMemoryFile.__init__(self, name, handle)
        self.type = FileRef.PROGRAM
    def get_name(self):
        return InMemoryFile.get_name(self) + ".akp"

class Sample(InMemoryFile):
    def __init__(self, name, handle):
        InMemoryFile.__init__(self, name, handle)
        self.type = FileRef.SAMPLE

    def get_name(self):
        return InMemoryFile.get_name(self) + ".wav"

    def copy(self, name):
        raise NotImplementedError("Copy not implemented for samples")
    
    def get_size(self):
        self.set_current()
        handlers[Sample].get_sample_length()
        
class Song(InMemoryFile):
    def __init__(self, name, handle):
        InMemoryFile.__init__(self, name, handle)
        self.type = FileRef.SONG

    def get_name(self):
        return InMemoryFile.get_name(self) + ".mid"
    
class Storage(Container):
    def __init__(self, name):
        self.name = name
        self.path = name
        self.actions = None
        self.type = FileRef.FOLDER
        self.children = []

    def refresh(self):
        del self.children[:]
        
    def has_children(self):
        return (len(self.children) > 0)

    def get_name(self):
        return self.name

    def get_short_name(self):
        return self.get_name()

    def get_handle(self):
        return self.name

    def get_children(self):
        return self.children

    def set_children(self, item_list):
        self.children = item_list

    def get_actions(self):
        # maybe implement an info action?
        return ()

class RootDisk(Storage):
    def __init__(self, name, disk_list):
        Storage.__init__(self, name)
        self.set_children([Disk(disk) for disk 
            in disk_list])

    def is_writable(self):
        return False

class Memory(Storage):
    def __init__(self, name):
        Storage.__init__(self, name)

    def is_writable(self):
        return True
    
    def get_actions(self):
        return ('upload',)
    
    def upload(self, path):
        name = os.path.basename(path)
        handlers[Disk].z48.put(path, name)
        item = InMemoryFile.get_instance(name)
        self.append_child(item)

    def append_child(self, item):
        self.children.append(item)
        return item

    def has_children(self):
        if len(self.children) > 0:
            return True
        return (handlers[Program].get_no_items() > 0 or
             handlers[Sample].get_no_items() > 0 or
             handlers[Song].get_no_items() > 0 or
             handlers[Multi].get_no_items() > 0)

    def get_children(self):
        if len(self.children) > 0:
            return self.children
        
        self.children.extend(self.get_handles_names(Program))
        self.children.extend(self.get_handles_names(Multi))
        self.children.extend(self.get_handles_names(Sample))
        self.children.extend(self.get_handles_names(Song))

        return self.children
    
    def get_handles_names(self, clz):
        handles_names = handlers[clz].get_handles_names()
        for i in range(0, len(handles_names), 2):
            handle, name = handles_names[i], handles_names[i+1]
            yield clz(name, handle)





import os, os.path, urlparse
import gtk
import urllib, thread 

from aksy.devices.akai import fileparser, sampler
from aksy import fileutils
from aksy import get_config

from aksui.utils import modelutils


try:
    from aksui.postmod import itx
except ImportError:
    print "postmod not available, no .it support"
    postmod_available = False

def get_file_path_from_dnd_dropped_uri(uri):
    path = uri.strip('\r\n\x00') # remove \r\n and NULL

    # get the path to file
    if path.startswith('file:\\\\\\'): # windows
        path = path[8:] # 8 is len('file:///')
    elif path.startswith('file://'): # nautilus, rox
        path = path[7:] # 7 is len('file://')
    elif path.startswith('file:'): # xffm
        path = path[5:] # 5 is len('file:')
        
    path = urllib.url2pathname(path) # escape special chars
    return path

def collect_files(args):
    collected = []
    for f in args:
        if os.path.isfile(f):
            if sampler.Sampler.is_filetype_supported(f):
                collected.append(f)
        elif os.path.isdir(f):
            collected.extend(collect_dir(f))
        else:
            raise IOError("File not found: " + repr(f))
    return collected

def collect_dir(args):
    for root, dir, files in os.walk(args):
        for found in files:
            if sampler.Sampler.is_filetype_supported(found):
                yield os.path.join(root, found)
 
def find_file(basedir, candidates, samplename):
    for f in candidates:
        name, ext = os.path.splitext(f)
        if samplename.lower() == name:
            return os.path.join(basedir, f)
    
    raise IOError("Referenced sample '%s' not found in directory '%s'" % (samplename, basedir))

def unwrap(func):
    return lambda(data): func(data[2])

class FileChooser:
    def __init__(self, s):
        self.s = s
        self.last_folder = None
        self.filechooser = gtk.FileChooserDialog(title="Open Sample", buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OPEN, gtk.RESPONSE_OK)) 
        self.setup_filter("All Sampler Supported Files", s.is_filetype_supported)
        self.setup_filter("Multis", fileutils.is_multi)
        self.setup_filter("Programs", fileutils.is_program)
        self.setup_filter("Samples", fileutils.is_sample)
        if postmod_available:
            self.setup_filter("Impulse Tracker Modules", extensions=["*.it", "*.IT"])
        self.filechooser.set_action(gtk.FILE_CHOOSER_ACTION_OPEN)

    def setup_filter(self, name, func=None, extensions=[]):
        """ takes a list of extensions and sets the filefilter
        """
        filter = gtk.FileFilter()
        filter.set_name(name)
         
        if func is not None:
            filter.add_custom(gtk.FILE_FILTER_DISPLAY_NAME, unwrap(func))
        else:
            for ext in extensions:
                filter.add_pattern(ext)
            
        self.filechooser.add_filter(filter)
       
    def open(self, multiple = True, upload = False, action = gtk.FILE_CHOOSER_ACTION_OPEN, title = "Upload files..."):
        self.filechooser.set_action(action)
        self.filechooser.set_select_multiple(multiple)
        self.filechooser.set_title(title)

        # multiple files (up to 4) will be distributed across zones (?)
        if self.last_folder:
            self.filechooser.set_current_folder(self.last_folder)
        else:
            self.filechooser.set_current_folder(get_config().get('DEFAULT', 'basedir'))

        response = self.filechooser.run()
        self.filechooser.hide()

        if response == gtk.RESPONSE_OK:
            self.updating = True
            if multiple:
                self.files = self.filechooser.get_filenames()
            else:
                self.files = [self.filechooser.get_filename(), ]
            
            if upload:
                self.upload_files()
                
            if action != gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER:
                nameonly = os.path.basename(self.files[0])[:-4]
            else:
                nameonly = self.files[0]

            self.last_folder = self.filechooser.get_current_folder()

            if multiple:
                return self.files
            else:
                return nameonly

        elif response == gtk.RESPONSE_CANCEL:
            self.files = None
            return None

    def import_from_it(self, fn):
        from aksui.utils import sox
        it = itx.ITX(fn)

        print fn, "loaded! exporting..."

        exported_files = it.exportSamples("c:\\tmp") # TODO: change to configurable temp dir

        resampled_path = None
        result = []

        for exported_file in exported_files:
            resampled_path = os.path.dirname(exported_file)
            resampled_name = sox.convert(exported_file)
            result.append(resampled_name)
                
            """
            if resampled_name:
                self.filechooser.upload(resampled_name, exported_file)
            else:
                pass
        if resampled_path:    
            print "Unlinking temp path", resampled_path
            print shutil.rmtree(resampled_path)
            """
            
        return result
    
    def upload_files(self):
        # hacky
        for f in self.files:
            if f:
                basedir = os.path.dirname(f)
                candidates = os.listdir(basedir)
                
                if fileutils.is_program(f):
                    filterProgram = f
                    filtered = []
                    program = fileparser.ProgramParser().parse(filterProgram)
                    for kg in program.keygroups:
                        for zone in kg.zones:
                            if zone.samplename:
                                filtered.append(find_file(basedir, candidates, zone.samplename))
                    self.files.extend(filtered)
                    
        already_done = []
        for f in self.files:
            if f and not f in already_done:
                self.s.transfertools.put(f)
                already_done.append(f)
                
        self.do_lists() 
            
    def do_lists(self):
        s = self.s
        setattr(s, 'samples', s.sampletools.get_names())
        setattr(s, 'programs', s.programtools.get_names())
        setattr(s, 'multis', s.multitools.get_names())

        setattr(s, 'samplesmodel', modelutils.get_model_from_list(s.samples))
        setattr(s, 'programsmodel', modelutils.get_model_from_list(s.programs))
        setattr(s, 'multismodel', modelutils.get_model_from_list(s.multis))
        
    def on_drag_data_received(self, widget, context, x, y, selection, target_type, timestamp):
        if target_type == 80: # TARGET_TYPE_URI_LIST
            uri = selection.data.strip()
            uris = uri.split() # we may have more than one file dropped
            files = []
            for uri in uris:
                path = get_file_path_from_dnd_dropped_uri(uri)
                if len(path):
                    print 'path to open', path
                    if os.path.isfile(path): # is it file?
                        files.append(path)
            if len(files) > 0:
                self.files = files
                thread.start_new_thread(self.upload_files)
                
    def expand_it_files(self, files):
        if files:
            additional_wavs = []
            for file in files:
                if file.lower().endswith('it'):
                    additional_wavs.extend(self.import_from_it(file))
                files.remove(file)
            files.extend(additional_wavs)
            print files
            return files
        else:
            return None



import gtk

import panelbase, rangewidget
from aksui.utils import midiutils

class FilterPanel(panelbase.PanelBase):
    def __init__(self,kg,cb):
        panelbase.PanelBase.__init__(self,kg,"Filter",cb)
        
    def setup(self, kg):
        self.clear_children(True)
        
        self.kg = kg
        self.s = kg.s

        vbox = gtk.VBox()
        hbox = gtk.HBox()

        controls = [
            rangewidget.AkComboBox(kg, "filter", midiutils.sampler_lists["filter"]),
            rangewidget.AkComboBox(kg, "filter_attenuation", midiutils.sampler_lists["filter_attenuation"]),
            ]
        controls_b = [
            rangewidget.AkKnobWidget(kg, "filter_cutoff", 0, 100, 1, ""), 
            rangewidget.AkKnobWidget(kg, "filter_resonance", 0, 100, 1, ""),
            ]

        for control in controls:
            hbox.pack_start(control, False, False, 1)
        #vbox.pack_start(hbox, False, False, 1)
        
        #hbox = gtk.HBox()
        for control in controls_b:
            hbox.pack_start(control, False, False, 1)
        vbox.pack_start(hbox, False, False, 1)
        self.pack_start(vbox, False, False, 1)
        self.show_all()



import struct, logging, types

START_SYSEX = '\xf0'
STRING_TERMINATOR = '\x00'
END_SYSEX = '\xf7'

POSTIVE     = '\x00'
NEGATIVE    = '\x01'

log = logging.getLogger("aksy.devices.akai.sysex_types")
log.disabled = True

def byte_repr(bytes):
    return repr([ "%02x" %byte for byte in struct.unpack(str(len(bytes)) + 'B', bytes)])

class SysexType(object):
    def __init__(self, size, signed=False, id=None):
        self.id = id
        self.size = size

        self.max_val = signed and pow(2, 7*size-1) - 1 or pow(2, 7*size) - 1
        self.min_val = signed and self.max_val*-1 or 0

    def validate_encode(self, value):
        if (value < self.min_val or
            value > self.max_val):
            raise ValueError("Value %s out of range:[%s-%s]" % (repr(value),
                              repr(self.min_val), repr(self.max_val)))

    def set_min_val(self, value):
        """
        Override for custom types
        """
        self.min_val = value
    def set_max_val(self, value):
        """
        Override for custom types
        """
        self.max_val = value

    def _encode(self, value):
        raise NotImplementedError("_encode")

    def _decode(self, value):
        raise NotImplementedError("_decode")
    
    def encode(self, value):
        """Encodes a value as sysex byte string
        """
        self.validate_encode(value)
        return self._encode(value)

    def decode(self, value, typed=True):
        """Decodes a value from a sysex byte string
        """
        if value is None or not isinstance(value, types.StringType):
            raise DecodeException(
                "Decoding error at %s.decode: %s is not a string"
                % (self.__class__.__name__, repr(value)))

        if self.size is not None:
            if len(value) == self.size + 1:
                if typed and value[0] != self.id:
                    raise DecodeException(
                        "Decoding error %s while decoding %s"
                            % (self.__class__.__name__, repr(value)))
                value = value[1:]
            elif self.size != len(value):
                raise DecodeException("Length of string to decode %s <> %s" % 
                          (repr(value), repr(self.size)))

        return self._decode(value)

class ByteType(SysexType):
    def __init__(self):
        SysexType.__init__(self, 1, False, '\x00')

    def _encode(self, value):
        """
        >>> b = ByteType()
        >>> b.encode(128)
        Traceback (most recent call last):
            if value > 127: raise ValueError
        ValueError: Value 128 out of range:[0-127]
        """

        return struct.pack('B', value)

    def _decode(self, string):
        """
        b = ByteType()
        b.decode('\x05')
        5
        """
        return struct.unpack('B', string)[0]

class SignedByteType(SysexType):
    def __init__(self):
        SysexType.__init__(self, 2, True, '\x01')

    def _encode(self, value):
        """
        sb = SignedByteType()
        sb.encode(-5)
        '\x01\x05'
        """
        sign = value < 0 and 1 or 0
        return struct.pack('2B', sign, abs(value))

    def _decode(self, string):
        """
        sb = SignedByteType()
        sb.decode('\x01\x05')
        -5
        """
        result = struct.unpack('B', string[1])[0]

        if string[0] == NEGATIVE:
            result *= -1
        return result

class WordType(SysexType):
    def __init__(self):
        SysexType.__init__(self, 2, False, '\x02')

    def _encode(self, value):
        return struct.pack('2B', value & 0x7f, value >> 7)

    def _decode(self, string):
        byte1, byte2 = struct.unpack('2B', string)
        return (byte2 << 7) + byte1

class CompoundWordType(SysexType):
    """ the little endian brother of WordType
    """
    def __init__(self):
        SysexType.__init__(self, 2, False, '\x02')

    def _encode(self, value):
        return struct.pack('2B', value >> 7,  value & 0x7f)

    def _decode(self, string):
        byte1, byte2 = struct.unpack('2B', string)
        return (byte1 << 7) + byte2


class SignedWordType(SysexType):
    def __init__(self):
        SysexType.__init__(self, 3, True, '\x03')

    def _encode(self, value):
        sign = value < 0 and 1 or 0
        value = abs(value)
        return struct.pack('3B', sign, value & 0x7f, value >> 7)

    def _decode(self, string):
        s, b1, b2 = struct.unpack('3B', string[:3])
        sign = s and -1 or 1
        return sign * ((b2 << 7) | b1)

class DoubleWordType(SysexType):
    def __init__(self):
        SysexType.__init__(self, 4, False, '\x04')

    def _encode(self, value):
        return struct.pack('4B', value & 0x7f, (value >> 7) & 0x7f, (value >> 14) & 0x7f, (value >> 21) & 0x7f)

    def _decode(self, string):
        b1, b2, b3, b4 = struct.unpack('4B', string)
        return (b4 << 21) | (b3 << 14) | (b2 << 7) | b1

class SignedDoubleWordType(SysexType):
    def __init__(self):
        SysexType.__init__(self, 5, True, '\x05')

    def _encode(self, value):
        sign = value < 0 and 1 or 0
        value = abs(value)
        return struct.pack('5B', sign, value & 0x7f, (value >> 7) & 0x7f, (value >> 14) & 0x7f, (value >> 21) & 0x7f)

    def _decode(self, string):
        s, b1, b2, b3, b4 = struct.unpack('5B', string)
        sign = s and -1 or 1
        return sign * ((b4 << 21) | (b3 << 14) | (b2 << 7) | b1)

class QWordType(SysexType):
    def __init__(self):
        SysexType.__init__(self, 8, False, '\x06')

    def _encode(self, value):
        return struct.pack('8B',  value & 0x7f, (value >> 7) & 0x7f,
            (value >> 14) & 0x7f, (value >> 21) & 0x7f, (value >> 28) & 0x7f,
            (value >> 35) & 0x7f, (value >> 42) & 0x7f, (value >> 49) & 0x7f)

    def _decode(self, string):
        b1, b2, b3, b4, b5, b6, b7, b8 = struct.unpack('8B', string)
        # bitshifting looks prettier but this works ;-)
        return (b8*pow(2,49)|b7*pow(2,42)|b6*pow(2,35)|b5*pow(2,28)|b4*pow(2,21)|b3*pow(2,14)|b2*pow(2,7)|b1)

class SignedQWordType(SysexType):
    def __init__(self):
        SysexType.__init__(self, 9, True, '\x07')

    def _encode(self, value):
        sign = value < 0 and 1 or 0
        value = abs(value)
        return struct.pack('9B', sign, value & 0x7f, (value >> 7) & 0x7f,
            (value >> 14) & 0x7f, (value >> 21) & 0x7f, (value >> 28) & 0x7f,
            (value >> 35) & 0x7f, (value >> 42) & 0x7f, (value >> 49) & 0x7f)

    def _decode(self, string):
        s, b1, b2, b3, b4, b5, b6, b7, b8 = struct.unpack('9B', string)
        sign = s and -1 or 1
        # bitshifting looks prettier but this works without losing bits ;-)
        return sign * (b8*pow(2,49)|b7*pow(2,42)|b6*pow(2,35)|b5*pow(2,28)|b4*pow(2,21)|b3*pow(2,14)|b2*pow(2,7)|b1)

class BoolType(ByteType):
    def __init__(self):
        ByteType.__init__(self)
        self.set_min_val(0)
        self.set_max_val(1)

    def _encode(self, value):
        return super(BoolType, self)._encode(int(value))

    def _decode(self, string):
        return bool(struct.unpack('B', string)[0])

class StringType(object):
    def __init__(self):
        self.id = '\x08'
        self.size = None # variable size, parsed length is returned in result

    def validate_encode(self, value):
        if not type(value) == type('') and value != '':
            raise ValueError("Value %s should be a (non unicode) string." % repr(value))

    def encode(self, value):
        self.validate_encode(value)
        return struct.pack(str(len(value)+1) + 's', value)

    def decode(self, string):
        index = string.find(STRING_TERMINATOR)
        if index == -1: 
            raise ValueError
        if string[0] == self.id:
            start = 1
        else:
            start = 0
        return index + 1, struct.unpack( str(index-start) + 's', string[start:index])[0]

class StringArrayType(object):
    def __init__(self):
        self.size = None # variable size, parsed length is returned in result

    def encode(self, value):
        raise NotImplementedError()

    def decode(self, string):
        if not string or not isinstance(string, types.StringType):
            raise DecodeException(
                "%s.decode: %s is not a string"
                % (self.__class__.__name__, repr(string)))

        result = []
        index = 0
        offset = 0
        for char in string:
            #sys.stderr.writelines( "c:%s i:%s\n" % (char, index) )
            if (char == END_SYSEX):
                break;
            if char == STRING_TERMINATOR:
                result.append(struct.unpack( str(index) + 's', string[offset:offset+index])[0])
                offset += index + 1
                index = 0
            else:
                index += 1
        return (offset + index, tuple(result))

class UserRefType(object):
    """ Can be used to stamp a request with an identifier
    size = None: variable size <= 2
    """
    def __init__(self, encoding_size=None):
        if encoding_size is not None and encoding_size > 3 and encoding_size < 0:
            raise ValueError("Illegal userref encoding_size %i" % encoding_size)
        self.encoding_size = encoding_size
        self.min_val = 0
        self.max_val = WORD.max_val

    def _encode_WORD(self, value):
        return BYTE.encode(2 << 4) + WORD.encode(value)

    def _encode_BYTE(self, value):
        return BYTE.encode(1 << 4) + BYTE.encode(value)

    def encode(self, value):
        if value < self.min_val or value > self.max_val:
            raise ValueError("Value %i out of range" % value)

        if self.encoding_size is None:
            if value == 0:
                return BYTE.encode(0)
            if value > BYTE.max_val:
                return self._encode_WORD(value)
            return self._encode_BYTE(value)

        if self.encoding_size == 0:
            return BYTE.encode(0)
        if self.encoding_size == 1:
            return self._encode_BYTE(value)
        if self.encoding_size == 2:
            return self._encode_WORD(value)

    def decode_length(self, byte):
        return byte >> 4

    def decode(self, string):
        if not string or not isinstance(string, types.StringType):
            raise ValueError(
                "Decoding error at %s.decode: %s is not a string"
                % (self.__class__.__name__, repr(string)))

        length = self.decode_length(BYTE.decode(string[0]))
        if length == 0:
            return (1, 0)
        if length == 1:
            return (2, BYTE.decode(string[1:2]))
        if length == 2:
            return (3, WORD.decode(string[1:3]))
        raise DecodeException("Unexpected UserRef length %i" % length)

class Z48UserRefType(UserRefType):

    def _encode_WORD(self, value):
        return BYTE.encode(2 << 5) + WORD.encode(value)

    def _encode_BYTE(self, value):
        return BYTE.encode(1 << 5) + BYTE.encode(value)

    def decode_length(self, byte):
        return byte >> 5

class TypeByteType(SysexType):
    def __init__(self):
        SysexType.__init__(self, 1, False)

    def _decode(self, value):
        return None

class TuneType(SignedWordType):
    """Represents tuning in cents
    """
    def __init__(self):
        SignedWordType.__init__(self)
        self.set_min_val(-3600)
        self.set_max_val(3600)

    def _encode(self, value):
        return super(TuneType, self)._encode(value)

    def _decode(self, string):
        return super(TuneType, self)._decode(string)

class SoundLevelType(SignedWordType):
    """Represents soundlevels in dB
    """
    def __init__(self):
        SignedWordType.__init__(self)
        self.set_min_val(-600)
        self.set_max_val(60)

    def _encode(self, value):
        return super(SoundLevelType, self)._encode(int(value*10))

    def _decode(self, string):
        return super(SoundLevelType, self)._decode(string)/10.0

class PanningType(ByteType):
    """Represents panning levels in -50->L, 50->R
    """

class FileType(ByteType):
    """Akai file types
    """
    ALL = 0
    MULTI = 1
    PROGRAM = 2
    SAMPLE = 3
    MIDI = 4
    def __init__(self):
        ByteType.__init__(self)
        self.set_min_val(0)
        self.set_max_val(4)

class CompositeType(object):
    def __init__(self, type_list):
        # log.debug("Initializing types %s" % repr(type_list))
        self.types = type_list

    def decode(self, string):
        offset = 0
        result = []
        for sysex_type in self.types:
            len_parsed, part = parse_byte_string(string, sysex_type, offset)
            if part is not None:
                result.append(part)
            offset += len_parsed
        if len(result) == 1:
            return offset, result[0]
        return offset, tuple(result)

class TypedCompositeType(object):
    def decode(self, string):
        result = []
        offset = 0
        while offset < (len(string) - 1):
            sysex_type = get_type(string[offset])
            if log.isEnabledFor(logging.DEBUG):
                log.debug("Parsing type %s" % sysex_type)
            offset += 1
            len_parsed, part = parse_byte_string(string, sysex_type, offset)
            if part is not None:
                result.append(part)
                offset += len_parsed
        if len(result) == 1:
            return offset, result[0]
        return offset, tuple(result)

class DiskType(CompositeType):
    def __init__(self):
        super(DiskType, self).__init__((
            WORD, BYTE,
            BYTE, BYTE,
            BYTE, STRING,))

class S56kDiskType(CompositeType):
    def __init__(self):
        super(S56kDiskType, self).__init__((
            CWORD, BYTE,
            BYTE, BYTE,
            BYTE, STRING,))


class DisklistType(object):
    def __init__(self):
        self.size = None

    def decode(self, string):
        result = []
        index = 0
        while string[index] != END_SYSEX:
            length, val = DISK.decode(string[index:])
            result.append(val)
            index += length
        return index, tuple(result)

class S56kDisklistType(object):
    def __init__(self):
        self.size = None

    def decode(self, string):
        result = []
        index = 0
        while string[index] != END_SYSEX:
            length, val = S56KDISK.decode(string[index:])
            result.append(val)
            index += length
        return index, tuple(result)


def parse_byte_string(data, sysex_type, offset=0):
    """ Parses a byte string
    """

    len_parsed_data = 0
    if sysex_type.size is not None:
        result = sysex_type.decode(data[offset:offset+sysex_type.size])
        len_parsed_data = sysex_type.size
    else:
        # TODO: use a factory which returns instances with a size set ?
        len_parsed_data, result = sysex_type.decode(data[offset:])
        if len_parsed_data == 0:
            result = None

    return len_parsed_data, result

class CompositeByteType(SysexType):
    def __init__(self, size, type_id):
        super(CompositeByteType, self).__init__(size, False, type_id)
        self.set_min_val(0)
        self.set_max_val(127)
    def encode(self, *args):
        if len(args) != self.size:
            raise ValueError("%s ints expected" % self.size)
        for byte in args:
            self.validate_encode(byte)

        return struct.pack("%iB" %self.size, *args)

    def _encode(self, string):
        pass # no-op, as encode itself is overridden
    
    def _decode(self, string):
        return struct.unpack("%iB" %self.size, string)

class TwoByteType(CompositeByteType):
    def __init__(self):
        super(TwoByteType, self).__init__(2, '\x09')

class ThreeByteType(CompositeByteType):
    def __init__(self):
        super(ThreeByteType, self).__init__(3, '\x0a')

class FourByteType(CompositeByteType):
    def __init__(self):
        super(FourByteType, self).__init__(4, '\x0b')

# TODO: make HandleNameDictType
class HandleNameArrayType(object):
    """Mixed data type, wrapping handle(DoubleWord) and name (StringType)
    """
    def __init__(self):
        self.size = None
    def encode(self, value):
        raise NotImplementedError

    def decode(self, string):
        results = []
        len_to_parse = len(string)
        len_parsed = 0
        while len_parsed < len_to_parse:
            handle = DWORD.decode(
                    string[len_parsed:len_parsed+5], typed=False)
            len_parsed += 5
            len_result, name = STRING.decode(string[len_parsed:])
            results.append((handle, name))
            len_parsed += len_result
        return len_parsed, tuple(results)

class NameSizeArrayType(object):
    """Mixed data type, wrapping name (StringType) and size (DoubleWord)
    """
    def __init__(self):
        self.size = None
    def encode(self, value):
        raise NotImplementedError

    def decode(self, string):
        results = []
        len_to_parse = len(string)
        len_parsed = 0
        while len_parsed < len_to_parse and string[len_parsed] != END_SYSEX:
            len_result, name = STRING.decode(string[len_parsed:])
            len_parsed += len_result
            size = DWORD.decode(
                    string[len_parsed-1:len_parsed+4], typed=False)
            len_parsed += 4
            results.append(name)
            results.append(size)
        return len_parsed, tuple(results)

# Base Sysex types
BYTE        = ByteType()
SBYTE       = SignedByteType()
WORD        = WordType()
SWORD       = SignedWordType()
DWORD       = DoubleWordType()
SDWORD      = SignedDoubleWordType()
QWORD       = QWordType()
SQWORD      = SignedQWordType()
STRING      = StringType()
STRINGARRAY = StringArrayType()

USERREF     = UserRefType()
Z48USERREF  = Z48UserRefType()

# S56k types
S56K_USERREF = UserRefType(2)
CWORD        = CompoundWordType()

TWO_BYTES   = TwoByteType()
THREE_BYTES = ThreeByteType()
FOUR_BYTES  = FourByteType()

# derived types
TYPEBYTE    = TypeByteType()
BOOL        = BoolType()
TUNE        = TuneType()
PAN         = PanningType()
FILETYPE    = FileType()
LEVEL       = SoundLevelType()

# Composite types
DISK        = DiskType()
DISKLIST    = DisklistType()
S56KDISK        = S56kDiskType()
S56KDISKLIST    = S56kDisklistType()
TYPED_COMPOSITE = TypedCompositeType()
HANDLENAMEARRAY = HandleNameArrayType()
NAMESIZEARRAY = NameSizeArrayType()

_types = {
    BYTE.id: BYTE,
    SBYTE.id: SBYTE,
    WORD.id: WORD,
    SWORD.id: SWORD,
    DWORD.id: DWORD,
    SDWORD.id: SDWORD,
    QWORD.id: QWORD,
    SQWORD.id: SQWORD,
    STRING.id: STRING,
    TWO_BYTES.id: TWO_BYTES,
    THREE_BYTES.id: THREE_BYTES,
    FOUR_BYTES.id: FOUR_BYTES,
}

class UnknownSysexTypeError(Exception):
    pass

class DecodeException(Exception):
    pass

def get_type(typeId):
    type = _types.get(typeId, None)
    if type is None:
        raise UnknownSysexTypeError( "%02x" % struct.unpack('B', typeId))
    return type




""" Python equivalent of akai section frontpaneltools

Methods to manipulate the front panel
"""

__author__ =  'Walco van Loon'
__version__ =  '0.2'

from aksy.devices.akai.sysex import Command

import aksy.devices.akai.sysex_types

class Frontpaneltools:
    def __init__(self, connector):
        self.connector = connector
        self.mouseclick_at_screen_cmd = Command('_', '\x2C\x20', 'frontpaneltools', 'mouseclick_at_screen', (aksy.devices.akai.sysex_types.WORD, aksy.devices.akai.sysex_types.WORD), None)
        self.mousedoubleclick_at_screen_cmd = Command('_', '\x2C\x21', 'frontpaneltools', 'mousedoubleclick_at_screen', (aksy.devices.akai.sysex_types.WORD, aksy.devices.akai.sysex_types.WORD), None)
        self.keypress_hold_cmd = Command('_', '\x2C\x01', 'frontpaneltools', 'keypress_hold', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.keypress_release_cmd = Command('_', '\x2C\x02', 'frontpaneltools', 'keypress_release', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.move_datawheel_cmd = Command('_', '\x2C\x03', 'frontpaneltools', 'move_datawheel', (aksy.devices.akai.sysex_types.SBYTE,), None)
        self.set_qlink_control_cmd = Command('_', '\x2C\x04', 'frontpaneltools', 'set_qlink_control', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.WORD), None)
        self.ascii_keypress_hold_cmd = Command('_', '\x2C\x10', 'frontpaneltools', 'ascii_keypress_hold', (aksy.devices.akai.sysex_types.WORD, aksy.devices.akai.sysex_types.WORD), None)
        self.ascii_keypress_release_cmd = Command('_', '\x2C\x11', 'frontpaneltools', 'ascii_keypress_release', (aksy.devices.akai.sysex_types.WORD, aksy.devices.akai.sysex_types.WORD), None)

        self.get_panel_state_cmd = aksy.devices.akai.sysex.Command('', '', 'frontpaneltools', 'get_panel_state', (), None)

    def mouseclick_at_screen(self, arg0, arg1):
        """Perform a mouse click
        """
        return self.connector.execute(self.mouseclick_at_screen_cmd, (arg0, arg1, ))

    def mousedoubleclick_at_screen(self, arg0, arg1):
        """Perform a mouse double click
        """
        return self.connector.execute(self.mousedoubleclick_at_screen_cmd, (arg0, arg1, ))

    def keypress_hold(self, arg0):
        """Perform key hold
        """
        return self.connector.execute(self.keypress_hold_cmd, (arg0, ))

    def keypress_release(self, arg0):
        """Perform key release
        """
        return self.connector.execute(self.keypress_release_cmd, (arg0, ))

    def move_datawheel(self, arg0):
        """Move datawheel
        """
        return self.connector.execute(self.move_datawheel_cmd, (arg0, ))

    def set_qlink_control(self, arg0, arg1):
        """Set the value of a Q-link control
        """
        return self.connector.execute(self.set_qlink_control_cmd, (arg0, arg1, ))

    def ascii_keypress_hold(self, arg0, arg1):
        """Perform ascii key hold
        """
        return self.connector.execute(self.ascii_keypress_hold_cmd, (arg0, arg1, ))

    def ascii_keypress_release(self, arg0, arg1):
        """Perform ascii key release
        """
        return self.connector.execute(self.ascii_keypress_release_cmd, (arg0, arg1, ))

    def get_panel_state(self):
        if hasattr(self.connector, 'get_panel_state'):
            return self.connector.get_panel_state()
        else:
            return self.connector.execute(self.get_panel_state_cmd, ())



import sys
import gtk, pango
from gettext import gettext as _
from cStringIO import *
import traceback

def _info(type, value, tb):
    dialog = gtk.MessageDialog(parent=None,
			       flags=0,
			       type=gtk.MESSAGE_WARNING,
			       buttons=gtk.BUTTONS_NONE,
			       message_format=_(
	"<big><b>A programming error has been detected during the execution of this program.</b></big>"
	"\n\nIt probably isn't fatal, but should be reported to the developers nonetheless."))
    dialog.set_title(_("Bug Detected"))
    dialog.set_property("has-separator", False)
    dialog.vbox.get_children()[0].get_children()[1].set_property("use-markup", True)

    dialog.add_button(_("Show Details"), 1)
    dialog.add_button(gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE)

    # Details
    textview = gtk.TextView(); textview.show()
    textview.set_editable(False)
    textview.modify_font(pango.FontDescription("Monospace"))
    sw = gtk.ScrolledWindow(); sw.show()
    sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
    sw.add(textview)
    frame = gtk.Frame();
    frame.set_shadow_type(gtk.SHADOW_IN)
    frame.add(sw)
    frame.set_border_width(6)
    dialog.vbox.add(frame)
    textbuffer = textview.get_buffer()
    trace = StringIO()
    traceback.print_exception(type, value, tb, None, trace)
    textbuffer.set_text(trace.getvalue())
    textview.set_size_request(gtk.gdk.screen_width()/2, gtk.gdk.screen_height()/3)

    dialog.details = frame
    dialog.set_position(gtk.WIN_POS_CENTER)
    dialog.set_gravity(gtk.gdk.GRAVITY_CENTER)
    
    while 1:
	resp = dialog.run()
	if resp == 1:
	    dialog.details.show()
	    dialog.action_area.get_children()[1].set_sensitive(0)
	else: break
    dialog.destroy()
    

sys.excepthook = _info





#!/usr/bin/python
import asyncore, socket, time

from aksyosc.handler import SamplerCallbackManager
from aksyosc.oscoptions import create_option_parser

class Envelope:
    def __init__(self, address, message):
        self.address = address
        self.message = message
    def get_message(self):
        return self.message
    def get_address(self):
        return self.address

class OSCServer(asyncore.dispatcher):
    def __init__(self, address, port, callbackManager):
        self._address = address
        self._port = port
        self._callbackMgr = callbackManager
        asyncore.dispatcher.__init__(self)
        self.create_socket(socket.AF_INET, socket.SOCK_DGRAM)
        self.set_reuse_addr()
        self.bind((address, port))
        self.response = None 
        print '%s started at addr: %s:%i\n' % (
           self.__class__.__name__, address, port)

    def handle_connect(self):
        pass
        
    def writable(self):
        return self.response is not None

    def handle_write(self):
        self.sendto(self.response.get_message(), 0, self.response.get_address())
        self.response = None

    def handle_read(self):
        data, address = self.recvfrom(8192)
        resp_data = self._callbackMgr.handle(data)
        self.response = Envelope(address, resp_data)
    
if __name__ == "__main__":
    parser = create_option_parser()
    options = parser.parse_args()[0]
    from aksy.device import Devices
    z48 = Devices.get_instance('mock_z48', None)
    server = OSCServer(options.address, options.port,  SamplerCallbackManager(z48))
    try:
        asyncore.loop()
    except KeyboardInterrupt:
        asyncore.close_all()




""" Python equivalent of akai section sampletools

Methods to manipulate in-memory samples
"""

__author__ =  'Walco van Loon'
__version__ =  '0.2'
__credits__ = ["Mike Goins"]

from aksy.devices.akai.sysex import Command

import aksy.devices.akai.sysex_types

class Sampletools:
    def __init__(self, s56k):
        self.sampler = s56k
        self.set_curr_by_name_cmd   = Command('^', '\x0e\x05', 'sampletools', 'set_curr_by_name', (aksy.devices.akai.sysex_types.STRING,), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.set_curr_by_handle_cmd = Command('^', '\x0e\x06', 'sampletools', 'set_curr_by_handle', (aksy.devices.akai.sysex_types.CWORD,), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.delete_all_cmd         = Command('^', '\x0e\x07', 'sampletools', 'delete_all', (), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.delete_curr_cmd        = Command('^', '\x0e\x08', 'sampletools', 'delete_curr', (), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.rename_curr_cmd        = Command('^', '\x0e\x09', 'sampletools', 'rename', (aksy.devices.akai.sysex_types.STRING,), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.play_cmd               = Command('^', '\x0e\x0a', 'sampletools', 'play', (), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.stop_cmd               = Command('^', '\x0e\x0b', 'sampletools', 'stop', (), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_no_items_cmd       = Command('^', '\x0e\x10', 'sampletools', 'get_no_items', (), (aksy.devices.akai.sysex_types.CWORD,), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_name_by_handle_cmd = Command('^', '\x0e\x11', 'sampletools', 'get_name_by_handle', (aksy.devices.akai.sysex_types.CWORD,), (aksy.devices.akai.sysex_types.STRINGARRAY,), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_names_cmd          = Command('^', '\x0e\x12', 'sampletools', 'get_names', (), (aksy.devices.akai.sysex_types.STRINGARRAY,), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_curr_handle_cmd    = Command('^', '\x0e\x13', 'sampletools', 'get_curr_handle', (), (aksy.devices.akai.sysex_types.CWORD,), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_curr_name_cmd      = Command('^', '\x0e\x14', 'sampletools', 'get_curr_name', (), (aksy.devices.akai.sysex_types.STRINGARRAY,), aksy.devices.akai.sysex_types.S56K_USERREF)
        

    def set_curr_by_name(self, arg0):
        """Select current item by name
        """
        return self.sampler.execute(self.set_curr_by_name_cmd, (arg0, ))

    def set_curr_by_handle(self, arg0):
        """Select current item by handle
        """
        return self.sampler.execute(self.set_curr_by_handle_cmd, (arg0, ))

    def delete_all(self):
        """Delete ALL items from memory
        """
        return self.sampler.execute(self.delete_all_cmd, ())

    def delete_curr(self):
        """Delete current item from memory
        """
        return self.sampler.execute(self.delete_curr_cmd, ())

    def rename(self):
        """Rename Sample
        """
        return self.sampler.execute(self.rename_curr_cmd, ())

    def play(self, arg0=127, arg1=0):
        """Start auditioning the current sample
        """
        return self.sampler.execute(self.play_cmd, ())

    def stop(self):
        """Stop playback of the current sample
        """
        return self.sampler.execute(self.stop_cmd, ())

    def get_no_items(self):
        """Get Number of Samples in Memory

        Returns:
            WORD
        """
        return self.sampler.execute(self.get_no_items_cmd, ())

    def get_name_by_handle(self, arg0):
        """Get item name from handle

        Returns:
            STRING
        """
        return self.sampler.execute(self.get_name_by_handle_cmd, (arg0, ))

    def get_names(self):
        """Get the Names of All Samples in memory

        Returns:
            STRINGARRAY
        """
        return self.sampler.execute(self.get_names_cmd, ())

    def get_curr_handle(self):
        """Current Sample Index (i.e., its position in memory)

        Returns:
            WORD
        """
        return self.sampler.execute(self.get_curr_handle_cmd, ())

    def get_curr_name(self):
        """Get Current Sample Name

        Returns:
            STRING
        """
        return self.sampler.execute(self.get_curr_name_cmd, ())


    def get_handles_names(self):
        """Get list of sample handles and names

        Returns:
            HANDLENAMEARRAY
        """
        handle_names = []
        names = self.get_names()
        for i in range(len(names)):
            handle_names.extend([i, names[i]])
        return handle_names

    def get_handle_by_name(self, arg0):
        """Get item handle from name

        Returns:
            DWORD
        """
        self.sampler.execute(self.set_curr_by_name_cmd, (arg0, ))
        return self.sampler.execute(self.get_curr_handle_cmd, ())

